@@ -37,13 +37,15 @@
  ##
 
 use strict;
+use warnings "all";
+no  warnings "uninitialized";
 
 use FindBin;
 use Cwd qw(getcwd abs_path);
 
-use lib "$FindBin::RealBin/lib";
-use lib "$FindBin::RealBin/lib/canu/lib/perl5";
-use lib "$FindBin::RealBin/lib/canu/lib64/perl5";
+use lib "$FindBin::RealBin/../lib/site_perl";
+#use lib "$FindBin::RealBin/lib/canu/lib/perl5";
+#use lib "$FindBin::RealBin/lib/canu/lib64/perl5";
 
 use File::Path 2.08 qw(make_path remove_tree);
 
@@ -54,15 +56,9 @@ use canu::Execution;
 
 use canu::Configure;
 
-use canu::Grid;
-use canu::Grid_Cloud;
-use canu::Grid_SGE;
-use canu::Grid_Slurm;
-use canu::Grid_PBSTorque;
-use canu::Grid_LSF;
-use canu::Grid_DNANexus;
+use canu::HaplotypeReads;
 
-use canu::Gatekeeper;
+use canu::SequenceStore;
 use canu::Meryl;
 use canu::OverlapInCore;
 use canu::OverlapMhap;
@@ -70,7 +66,6 @@ use canu::OverlapMMap;
 use canu::OverlapStore;
 
 use canu::CorrectReads;
-use canu::ErrorEstimate;
 
 use canu::OverlapBasedTrimming;
 
@@ -79,20 +74,25 @@ use canu::Unitig;
 use canu::Consensus;
 use canu::Output;
 
-use canu::HTML;
+use canu::Grid;
+use canu::Grid_Cloud;
+use canu::Grid_SGE;
+use canu::Grid_Slurm;
+use canu::Grid_PBSTorque;
+use canu::Grid_LSF;
+use canu::Grid_DNANexus;
 
-my @specFiles;    #  Files of specs
-my @specOpts;     #  Command line specs
-my @inputFiles;   #  Command line inputs, later inputs in spec files are added
+my @specFiles;       #  Files of specs
+my @specOpts;        #  Command line specs
+my @inputFiles;      #  Command line inputs, later inputs in spec files are added
+
+my %haplotypeReads;  #  Inpout reads for haplotypes; each element is a NUL-delimited list of files
 
 #  Initialize our defaults.  Must be done before defaults are reported in printOptions() below.
 
 setDefaults();
 
-#  The bin directory is needed for -version, can only be set after setDefaults(), but really should be
-#  set after checkParameters() so it can know pathMap.
-
-my $bin     = getBinDirectory();  #  Path to binaries, reset later.
+my $bin     = getBinDirectory();  #  Path to binaries, must be set after setDefaults().
 my $cmd     = undef;              #  Temporary string passed to system().
 my $asm     = undef;              #  Name of our assembly.
 my $asmAuto = undef;              #  If set, the name was auto-discovered.
@@ -104,8 +104,10 @@ my $asmAuto = undef;              #  If set, the name was auto-discovered.
 
 setVersion($bin);
 
-#  Check for the presence of a -options switch BEFORE we do any work.
-#  This lets us print the default values of options.
+#  Check for the presence of -option switches BEFORE we do any work.
+#  This lets us print the default values of options (which we don't do anymore, because
+#  too many are set later), and change the defaults (-fast and -slow) before
+#  other options are applied.
 
 if (scalar(@ARGV) == 0) {
     printHelp(1);
@@ -123,6 +125,22 @@ foreach my $arg (@ARGV) {
         print getGlobal("version") . "\n";
         exit(0);
     }
+
+    if ($arg eq "-fast") {
+        #  All defaults, unless noted.
+        setGlobal("corOverlapper",  "mhap");
+        setGlobal("obtOverlapper",  "mhap");    # Changed
+        setGlobal("utgOverlapper",  "mhap");    # Changed
+        setGlobal("utgReAlign",        "true");    # Changed
+
+        if (-e "$bin/wtdbg-1.2.8") {
+           setGlobal("unitigger",      "wtdbg");   # Changed
+        }
+    }
+
+    if ($arg eq "-accurate") {
+        #  All defaults, unless noted.
+    }
 }
 
 #  By default, all three steps are run.  Options -correct, -trim and -assemble
@@ -137,8 +155,6 @@ my $readdir       = undef;
 my $mode          = undef;   #  "correct", "trim", "trim-assemble" or "assemble"
 my $type          = undef;   #  "pacbio" or "nanopore"
 my $step          = "run";
-my $haveRaw       = 0;
-my $haveCorrected = 0;
 
 while (scalar(@ARGV)) {
     my $arg = shift @ARGV;
@@ -146,6 +162,10 @@ while (scalar(@ARGV)) {
     if     (($arg eq "-h") || ($arg eq "-help") || ($arg eq "--help")) {
         printHelp(1);
 
+    } elsif (($arg eq "-fast") ||
+             ($arg eq "-accurate")) {
+        addCommandLineOption($arg);
+
     } elsif (($arg eq "-citation") || ($arg eq "--citation")) {
         print STDERR "\n";
         printCitation(undef);
@@ -186,14 +206,9 @@ while (scalar(@ARGV)) {
         $readdir = shift @ARGV;
         addCommandLineOption("-readdir '$readdir'");
 
-    } elsif (($arg eq "-pacbio") ||
-             ($arg eq "-nanopore")) {
-        $type = "pacbio"    if ($arg eq "-pacbio");
-        $type = "nanopore"  if ($arg eq "-nanopore");
-
-    } elsif (($arg eq "-pacbio-raw")       ||    #  File handling is also present in
-             ($arg eq "-pacbio-corrected") ||    #  Defaults.pm around line 438
-             ($arg eq "-nanopore-raw")     ||
+    } elsif (($arg eq "-pacbio-raw")         ||  #  File handling is also present in Defaults.pm,
+             ($arg eq "-pacbio-corrected")   ||  #  look for addSequenceFile().
+             ($arg eq "-nanopore-raw")       ||
              ($arg eq "-nanopore-corrected")) {
 
         my $file = $ARGV[0];
@@ -209,8 +224,24 @@ while (scalar(@ARGV)) {
             $fopt = addSequenceFile($readdir, $file);
         }
 
+    } elsif ($arg =~ m/^-haplotype(\w+)$/) {
+        my $hapn = $1;
+        my $file = $ARGV[0];
+        my $fopt = addSequenceFile($readdir, $file, 1);
+
+        while (defined($fopt)) {
+            $haplotypeReads{$hapn} .= "$fopt\0";
+
+            addCommandLineOption("$arg '$fopt'");
+
+            shift @ARGV;
+
+            $file = $ARGV[0];
+            $fopt = addSequenceFile($readdir, $file);
+        }
+
     } elsif (-e $arg) {
-        addCommandLineError("ERROR:  File supplied on command line; use -s, -pacbio-raw, -pacbio-corrected, -nanopore-raw, or -nanopore-corrected.\n");
+        addCommandLineError("ERROR:  File '$arg' supplied on command line, don't know what to do with it.\n");
 
     } elsif ($arg =~ m/=/) {
         push @specOpts, $arg;
@@ -226,11 +257,9 @@ while (scalar(@ARGV)) {
 if (!defined($asm)) {
     $asmAuto = 1;   #  If we don't actually find a prefix, we'll fail right after this, so OK to set blindly.
 
-    open(F, "ls -d */*gkpStore |");
+    open(F, "ls -d . *seqStore |");
     while (<F>) {
-        $asm = $1   if (m/^correction\/(.*).gkpStore$/);
-        $asm = $1   if (m/^trimming\/(.*).gkpStore$/);
-        $asm = $1   if (m/^unitigging\/(.*).gkpStore$/);
+        $asm = $1   if (m/^(.*).seqStore$/);
     }
     close(F);
 }
@@ -254,10 +283,6 @@ foreach my $specFile (@specFiles) {
 
 setParametersFromCommandLine(@specOpts);
 
-#  Reset $bin, now that all options, specifically the pathMap, are set.
-
-$bin = getBinDirectory();
-
 #  If anything complained (invalid option, missing file, etc) printHelp() will trigger and exit.
 
 printHelp();
@@ -276,6 +301,7 @@ print STDERR "--\n";
 #  Check java and gnuplot.
 
 checkJava();
+checkMinimap($bin);
 checkGnuplot();
 
 #  And one last chance to fail - because java and gnuplot both can set an error.
@@ -316,61 +342,100 @@ configureSlurm();
 configurePBSTorque();
 configureLSF();
 configureRemote();
+configureCloud($asm);
 configureDNANexus();
 
-#  Based on genomeSize, configure the execution of every component.
-#  This needs to be done AFTER the grid is setup!
+#  Set jobs sizes based on genomeSize and available hosts;
+#  Check that parameters (except error rates) are valid and consistent;
+#  Fail if any thing flagged an error condition;
 
-configureAssembler();
+configureAssembler();  #  Set job sizes and etc bases on genomeSize and hosts available.
+checkParameters();     #  Check all parameters (except error rates) are valid and consistent.
+printHelp();           #  And one final last chance to fail.
 
-#  And, finally, move to the assembly directory, finish setting things up, and report the critical
-#  parameters.
+#  Make space for us to work in, and move there.
 
-setWorkDirectory();
-
-if (defined($rootdir)) {
-    make_path($rootdir)  if (! -d $rootdir);
-    chdir($rootdir);
-}
-
-setGlobal("onExitDir", getcwd());
-setGlobal("onExitNam", $asm);
-
-setGlobalIfUndef("objectStoreNameSpace", $asm);   #  No good place to put this.
+setWorkDirectory($asm, $rootdir);
 
 #  Figure out read inputs.  From an existing store?  From files?  Corrected?  Etc, etc.
 
-my $haveCorrected    = 0;
 my $haveRaw          = 0;
+my $haveCorrected    = 0;
 
 my $setUpForPacBio   = 0;
 my $setUpForNanopore = 0;
 
-#  If we're a cloud run, fetch the store we expect to be working with.
+#  If we're a cloud run, fetch the store.
 
-fetchStore("unitigging/$asm.gkpStore")    if ((! -e "unitigging/$asm.gkpStore") && (fileExists("unitigging/$asm.gkpStore.tar")));
-fetchStore("trimming/$asm.gkpStore")      if ((! -e "trimming/$asm.gkpStore")   && (fileExists("trimming/$asm.gkpStore.tar"))   && (! -e "unitigging/$asm.gkpStore"));
-fetchStore("correction/$asm.gkpStore")    if ((! -e "correction/$asm.gkpStore") && (fileExists("correction/$asm.gkpStore.tar")) && (! -e "trimming/$asm.gkpStore"));
+fetchSeqStore($asm);
 
-#  Scan for an existing gkpStore.  If the output from that stage exists, ignore the store there.
+#  Scan for an existing seqStore and decide what reads and types are in it.  This is
+#  pretty horrible and needs to be consolidated into a single report.
 
-my $gkp;
+my $nCor = getNumberOfReadsInStore($asm, "cor");   #  Number of raw reads ready for correction.
+my $nOBT = getNumberOfReadsInStore($asm, "obt");   #  Number of corrected reads ready for OBT.
+my $nAsm = getNumberOfReadsInStore($asm, "utg");   #  Number of trimmed reads ready for assembly.
 
-$gkp = "correction/$asm.gkpStore"   if ((-e "correction/$asm.gkpStore/libraries.txt")  && (sequenceFileExists("$asm.correctedReads") eq undef));
-$gkp = "trimming/$asm.gkpStore"     if ((-e "trimming/$asm.gkpStore/libraries.txt")    && (sequenceFileExists("$asm.trimmedReads")   eq undef));
-$gkp = "unitigging/$asm.gkpStore"   if ((-e "unitigging/$asm.gkpStore/libraries.txt"));
+#  If a seqStore was found, scan the reads in it to decide what we're working with.
+#  The sqStoreDumpMetaData here isn't used normally; it's only used when canu runs
+#  on a seqStore created by hand.
 
-#  Scan for existing stage outputs.  These only get used if there isn't a gkpStore found above.
+if ($nCor + $nOBT + $nAsm > 0) {
+    my $numPacBioRaw         = 0;
+    my $numPacBioCorrected   = 0;
+    my $numNanoporeRaw       = 0;
+    my $numNanoporeCorrected = 0;
 
-my $reads;
+    if (! -e "./$asm.seqStore/libraries.txt") {
+        if (runCommandSilently(".", "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -libs > ./$asm.seqStore/libraries.txt 2> /dev/null", 1)) {
+            caExit("failed to generate list of libraries in store", undef);
+        }
+    }
 
-$reads = sequenceFileExists("$asm.correctedReads")  if (!defined($reads));
-$reads = sequenceFileExists("$asm.trimmedReads")    if (!defined($reads));
+    open(L, "< ./$asm.seqStore/libraries.txt") or caExit("can't open './$asm.seqStore/libraries.txt' for reading: $!", undef);
+    while (<L>) {
+        my @v = split '\s+', $_;
 
-#  A handy function for reporting what reads we found.
+        $numPacBioRaw++           if ($v[2] eq "pacbio-raw");
+        $numPacBioCorrected++     if ($v[2] eq "pacbio-corrected");
+        $numNanoporeRaw++         if ($v[2] eq "nanopore-raw");
+        $numNanoporeCorrected++   if ($v[2] eq "nanopore-corrected");
+    }
+    close(L);
 
-sub reportReadsFound ($$$$) {
-    my ($setUpForPacBio, $setUpForNanopore, $haveRaw, $haveCorrected) = @_;
+    $setUpForPacBio++      if ($numPacBioRaw       + $numPacBioCorrected   > 0);
+    $setUpForNanopore++    if ($numNanoporeRaw     + $numNanoporeCorrected > 0);
+
+    $haveRaw++             if ($numPacBioRaw       + $numNanoporeRaw       > 0);
+    $haveCorrected++       if ($numPacBioCorrected + $numNanoporeCorrected > 0);
+
+    my $rt;
+
+    $rt = "both PacBio and Nanopore"    if (($setUpForPacBio  > 0) && ($setUpForNanopore  > 0));
+    $rt = "PacBio"                      if (($setUpForPacBio  > 0) && ($setUpForNanopore == 0));
+    $rt = "Nanopore"                    if (($setUpForPacBio == 0) && ($setUpForNanopore  > 0));
+
+    #my $rtct = reportReadsFound($setUpForPacBio, $setUpForNanopore, $haveRaw, $haveCorrected);
+
+    print STDERR "--\n";
+    print STDERR "-- In '$asm.seqStore', found $rt reads:\n";
+    print STDERR "--   Raw:        $nCor\n";
+    print STDERR "--   Corrected:  $nOBT\n";
+    print STDERR "--   Trimmed:    $nAsm\n";
+}
+
+#  Otherwise, scan input files, counting the different types of libraries we have.
+
+elsif (scalar(@inputFiles) > 0) {
+    foreach my $typefile (@inputFiles) {
+        my ($type, $file) = split '\0', $typefile;
+
+        $haveCorrected++         if ($type =~ m/corrected/);
+        $haveRaw++               if ($type =~ m/raw/);
+
+        $setUpForPacBio++        if ($type =~ m/pacbio/);
+        $setUpForNanopore++      if ($type =~ m/nanopore/);
+    }
 
     my $rt;
     my $ct;
@@ -384,106 +449,28 @@ sub reportReadsFound ($$$$) {
     $ct = "corrected"                   if (($haveRaw        == 0) && ($haveCorrected     > 0));
     $ct = "uncorrected AND corrected"   if (($haveRaw         > 0) && ($haveCorrected     > 0));
 
-    return("$rt $ct");
-}
-
-#  If a gkpStore was found, scan the reads in it to decide what we're working with.
-
-if (defined($gkp)) {
-    my $numPacBioRaw         = 0;
-    my $numPacBioCorrected   = 0;
-    my $numNanoporeRaw       = 0;
-    my $numNanoporeCorrected = 0;
-
-    open(L, "< $gkp/libraries.txt") or caExit("can't open '$gkp/libraries.txt' for reading: $!", undef);
-    while (<L>) {
-        $numPacBioRaw++           if (m/pacbio-raw/);
-        $numPacBioCorrected++     if (m/pacbio-corrected/);
-        $numNanoporeRaw++         if (m/nanopore-raw/);
-        $numNanoporeCorrected++   if (m/nanopore-corrected/);
-    }
-    close(L);
-
-    $setUpForPacBio++      if ($numPacBioRaw       + $numPacBioCorrected   > 0);
-    $setUpForNanopore++    if ($numNanoporeRaw     + $numNanoporeCorrected > 0);
-
-    $haveRaw++             if ($numPacBioRaw       + $numNanoporeRaw       > 0);
-    $haveCorrected++       if ($numPacBioCorrected + $numNanoporeCorrected > 0);
-
-    my $rtct = reportReadsFound($setUpForPacBio, $setUpForNanopore, $haveRaw, $haveCorrected);
+    #my $rtct = reportReadsFound($setUpForPacBio, $setUpForNanopore, $haveRaw, $haveCorrected);
 
     print STDERR "--\n";
-    print STDERR "-- Found $rtct reads in '$gkp'.\n";
+    print STDERR "-- Found $rt $ct reads in the input files.\n";
 }
 
-#  Like above, scan the gkpStore to decide what we're working with.  The catch here is that
-#  we scan the previous store, and all reads are corrected.
+#  Otherwise, no reads found in a store, and no input files.
 
-elsif (defined($reads)) {
-
-    $gkp = "correction/$asm.gkpStore"   if ((-e "correction/$asm.gkpStore/libraries.txt")  && (sequenceFileExists("$asm.correctedReads")));
-    $gkp = "trimming/$asm.gkpStore"     if ((-e "trimming/$asm.gkpStore/libraries.txt")    && (sequenceFileExists("$asm.trimmedReads")));
-
-    my $numPacBio   = 0;
-    my $numNanopore = 0;
-
-    if (defined($gkp)) {
-        open(L, "< $gkp/libraries.txt") or caExit("can't open '$gkp/libraries.txt' for reading: $!", undef);
-        while (<L>) {
-            $numPacBio++           if (m/pacbio/);
-            $numNanopore++         if (m/nanopore/);
-        }
-        close(L);
-
-        $setUpForPacBio++      if ($numPacBio    > 0);
-        $setUpForNanopore++    if ($numNanopore  > 0);
-
-        $haveCorrected++;
-    } else {
-        #$setUpForPacBio++;   #  Leaving both setUp's as zero reports 'unknown' and
-        $haveCorrected++;     #  defaults to Pacbio below (search for setUpForNanopore).
-    }
-
-    #  Regardless of what the user gave us, we always want to restart with these reads.
-
-    undef @inputFiles;
-    push  @inputFiles, (($setUpForNanopore == 0) ? "-pacbio" : "-nanopore") . "-corrected\0$reads";
-
-    my $rtct = reportReadsFound($setUpForPacBio, $setUpForNanopore, $haveRaw, $haveCorrected);
-
-    print STDERR "--\n";
-    print STDERR "-- Found $rtct reads in '$reads'.\n";
-}
-
-#  Scan input files, counting the different types of libraries we have.
-
-elsif (scalar(@inputFiles) > 0) {
-    foreach my $typefile (@inputFiles) {
-        my ($type, $file) = split '\0', $typefile;
-
-        $haveCorrected++         if ($type =~ m/corrected/);
-        $haveRaw++               if ($type =~ m/raw/);
-
-        $setUpForPacBio++        if ($type =~ m/pacbio/);
-        $setUpForNanopore++      if ($type =~ m/nanopore/);
-    }
-
-    my $rtct = reportReadsFound($setUpForPacBio, $setUpForNanopore, $haveRaw, $haveCorrected);
-
-    print STDERR "--\n";
-    print STDERR "-- Found $rtct reads in the input files.\n";
+else {
+    caExit("ERROR: No reads supplied, and can't find any reads in any seqStore", undef);
 }
 
 #  Set an initial run mode, based on the libraries we have found, or the stores that exist (unless
 #  it was set on the command line).
 
 if (!defined($mode)) {
-    $mode = "run"            if ($haveRaw       > 0);
-    $mode = "trim-assemble"  if ($haveCorrected > 0);
+    $mode = "run"            if ($haveRaw       > 0);   #  If no seqStore, these are set based
+    $mode = "trim-assemble"  if ($haveCorrected > 0);   #  on flags describing the input files.
 
-    $mode = "run"            if (-e "correction/$asm.gkpStore/libraries.txt");
-    $mode = "trim-assemble"  if (-e "trimming/$asm.gkpStore/libraries.txt");
-    $mode = "assemble"       if (-e "unitigging/$asm.gkpStore/libraries.txt");
+    $mode = "run"            if ($nCor > 0);            #  If a seqStore, these are set based
+    $mode = "trim-assemble"  if ($nOBT > 0);            #  on the reads present in the stores.
+    $mode = "assemble"       if ($nAsm > 0);
 }
 
 #  Set the type of the reads.  A command line option could force the type, e.g., "-pacbio" or
@@ -498,12 +485,12 @@ if (!defined($type)) {
 
 if ($type eq"nanopore") {
     setGlobalIfUndef("corOvlErrorRate",  0.320);
-    setGlobalIfUndef("obtOvlErrorRate",  0.144);
-    setGlobalIfUndef("utgOvlErrorRate",  0.144);
+    setGlobalIfUndef("obtOvlErrorRate",  0.120);
+    setGlobalIfUndef("utgOvlErrorRate",  0.120);
     setGlobalIfUndef("corErrorRate",     0.500);
-    setGlobalIfUndef("obtErrorRate",     0.144);
-    setGlobalIfUndef("utgErrorRate",     0.144);
-    setGlobalIfUndef("cnsErrorRate",     0.192);
+    setGlobalIfUndef("obtErrorRate",     0.120);
+    setGlobalIfUndef("utgErrorRate",     0.120);
+    setGlobalIfUndef("cnsErrorRate",     0.200);
 }
 
 if ($type eq"pacbio") {
@@ -520,18 +507,10 @@ if ($type eq"pacbio") {
 #    no mode                -> don't have any reads or any store to run from.
 #    both raw and corrected -> don't know how to process these
 
-caExit("ERROR: No reads supplied, and can't find any reads in any gkpStore", undef)   if (!defined($mode));
+caExit("ERROR: No reads supplied, and can't find any reads in any seqStore", undef)   if (!defined($mode));
 caExit("ERROR: Failed to determine the sequencing technology of the reads", undef)    if (!defined($type));
 caExit("ERROR: Can't mix uncorrected and corrected reads", undef)                     if ($haveRaw && $haveCorrected);
 
-#  Do a final check on parameters, cleaning up paths and case, and failing on invalid stuff.
-
-checkParameters();
-
-#  And one final last chance to fail - because java and gnuplot both can set an error.
-
-printHelp();
-
 #  Go!
 
 printf STDERR "--\n";
@@ -565,11 +544,6 @@ $ENV{'CANU_DIRECTORY'} = getcwd();
 
 writeLog();
 
-#  Submit ourself for grid execution?  If not grid enabled, or already running on the grid, this
-#  call just returns.  The arg MUST be undef.
-
-submitScript($asm, undef);
-
 #
 #  When doing 'run', this sets options for each stage.
 #    - overlapper 'mhap' for correction, 'ovl' for trimming and assembly.
@@ -596,9 +570,10 @@ sub setOptions ($$) {
 
     #  Create directories for the step, if needed.
 
-    make_path("correction")  if ((! -d "correction") && ($step eq "correct"));
-    make_path("trimming")    if ((! -d "trimming")   && ($step eq "trim"));
-    make_path("unitigging")  if ((! -d "unitigging") && ($step eq "assemble"));
+    make_path("haplotype")    if ((! -d "haplotype")   && ($step eq "haplotype"));
+    make_path("correction")   if ((! -d "correction")  && ($step eq "correct"));
+    make_path("trimming")     if ((! -d "trimming")    && ($step eq "trim"));
+    make_path("unitigging")   if ((! -d "unitigging")  && ($step eq "assemble"));
 
     #  Return that we want to run this step.
 
@@ -617,141 +592,289 @@ sub overlap ($$) {
 
     if (getGlobal("${tag}overlapper") eq "mhap") {
         mhapConfigure($asm, $tag, $ovlType);
-
         mhapPrecomputeCheck($asm, $tag, $ovlType)  foreach (1..getGlobal("canuIterationMax") + 1);
-
-        #  this also does mhapReAlign
-
-        mhapCheck($asm, $tag, $ovlType)  foreach (1..getGlobal("canuIterationMax") + 1);
+        mhapCheck($asm, $tag, $ovlType)            foreach (1..getGlobal("canuIterationMax") + 1);          #  this also does mhapReAlign
 
    } elsif (getGlobal("${tag}overlapper") eq "minimap") {
         mmapConfigure($asm, $tag, $ovlType);
-
         mmapPrecomputeCheck($asm, $tag, $ovlType)  foreach (1..getGlobal("canuIterationMax") + 1);
-
-        mmapCheck($asm, $tag, $ovlType)   foreach (1..getGlobal("canuIterationMax") + 1);
+        mmapCheck($asm, $tag, $ovlType)            foreach (1..getGlobal("canuIterationMax") + 1);
 
     } else {
         overlapConfigure($asm, $tag, $ovlType);
-
-        overlapCheck($asm, $tag, $ovlType)  foreach (1..getGlobal("canuIterationMax") + 1);
+        overlapCheck($asm, $tag, $ovlType)         foreach (1..getGlobal("canuIterationMax") + 1);
     }
 
-    createOverlapStore($asm, $tag, getGlobal("ovsMethod"));
+    createOverlapStore($asm, $tag);
 }
 
 #
 #  Begin pipeline
 #
-#  The checks for sequenceFileExists() at the start aren't needed except for
-#  object storage mode.  Gatekeeper has no way of knowing, inside
-#  gatekeeper(), that this stage is completed and it shouldn't fetch the
-#  store.  In 'normal' operation, the store exists already, and we just
-#  return.
+
+my @haplotypes = sort keys %haplotypeReads;
+
+if (setOptions($mode, "haplotype") eq "haplotype") {
+    if ((! -e "./haplotype/haplotyping.success") &&
+        (haplotypeReadsExist($asm, @haplotypes) eq "no")) {
+
+        submitScript($asm, undef);   #  See comments there as to why this is safe.
+
+        print STDERR "--\n";
+        print STDERR "--\n";
+        print STDERR "-- BEGIN HAPLOTYPING\n";
+        print STDERR "--\n";
+
+        haplotypeCountConfigure($asm, %haplotypeReads);
+
+        haplotypeCountCheck($asm)                   foreach (1..getGlobal("canuIterationMax") + 1);
+        haplotypeMergeCheck($asm, @haplotypes)      foreach (1..getGlobal("canuIterationMax") + 1);
+        haplotypeSubtractCheck($asm, @haplotypes)   foreach (1..getGlobal("canuIterationMax") + 1);
+
+        haplotypeReadsConfigure($asm, \@haplotypes, \@inputFiles);
+        haplotypeReadsCheck($asm)                   foreach (1..getGlobal("canuIterationMax") + 1);
+    }
+}
+
+#  If haplotype reads exist, bootstrap the assemblies.
 #
+#  I tried to use submitScript() to launch these, but that didn't work
+#  so nicely - if not on grid, it wouldn't do anything.
+
+if (haplotypeReadsExist($asm, @haplotypes) eq "yes") {
+    my $techtype = removeHaplotypeOptions();
+    my @options  = getCommandLineOptions();
+
+    #  Find the maximum length of haplotype names, to make the output pretty.
+
+    my $displLen = 0;
+
+    foreach my $haplotype (@haplotypes) {
+        my $hapLen = length($haplotype);
+        $displLen = ($displLen < $hapLen) ? $hapLen : $displLen;
+    }
+
+    #  Decide if we should use or ignore the unassigned reads, and if we should
+    #  even bother assembling.
+
+    fetchFile("");
+
+    my %hapReads;
+    my %hapBases;
+
+    my $totReads = 0;
+    my $totBases = 0;
+
+    open(F, "< haplotype/haplotype.log") or caExit("can't open 'haplotype/haplotype.log' for reading: $!", undef);
+    while (<F>) {
+        if (m/(\d+)\s+reads\s+(\d+)\s+bases\s+written\s+to\s+haplotype\s+file\s+.*haplotype-(\w+).fasta.gz/) {
+            $hapReads{$3} = $1;     $totReads += $1;
+            $hapBases{$3} = $2;     $totBases += $2;
+        }
+    }
+    close(F);
+
+    print STDERR "--\n";
+    foreach my $haplotype (@haplotypes) {
+        printf STDERR "-- Found %8d reads and %12d bases for haplotype $haplotype.\n", $hapReads{$haplotype}, $hapBases{$haplotype};
+    }
+    printf STDERR "-- Found %8d reads and %12d bases assigned to no haplotype.\n", $hapReads{"unknown"}, $hapBases{"unknown"};
+
+    #  Ignore the unknown reads if there aren't that many.
+
+    my $withUnknown = ($hapBases{"unknown"} / $totBases < 0.02) ? 0 : 1;
+
+    if ($withUnknown == 0) {
+        print STDERR "--\n";
+        print STDERR "-- Too few bases in unassigned reads to care; don't use them in assemblies.\n";
+    }
+
+
+    #  For each haplotype, emit a script to run the assembly.
+
+    print STDERR "--\n";
+
+    foreach my $haplotype (@haplotypes) {
+        my $hs = substr("$haplotype" . " " x $displLen, 0, $displLen);
+
+        print STDERR "-- Assemble haplotype $hs with command './$asm-haplotype$haplotype.sh'.\n";
+
+        open(F, "> ./$asm-haplotype$haplotype.sh");
+        print F "#!/bin/sh\n";
+        print F "\n";
+
+        if (defined(getGlobal("objectStore"))) {
+            print F "\n";
+            print F "#  Fetch the haplotyped reads.  This is just a bit weird.\n";
+            print F "#  The fetch (boilerplate) only works from within a subdirectory,\n";
+            print F "#  so we must cd into it first, fetch, the go back to the root.\n";
+            print F "\n";
+            print F "mkdir -p haplotype\n";
+            print F "cd       haplotype\n";
+            print F fetchFileShellCode("haplotype", "haplotype-$haplotype.fasta.gz", "");
+            print F fetchFileShellCode("haplotype", "haplotype-unnown.fasta.gz", "")      if ($withUnknown);
+            print F "cd ..\n";
+        }
+
+        print F "\n";
+        print F "$bin/canu \\\n";
+        print F "  -p $asm-haplotype$haplotype \\\n";
+        print F "  -d $asm-haplotype$haplotype \\\n";
+        print F "  $_ \\\n"   foreach (@options);
+        print F "  $techtype ./haplotype/haplotype-$haplotype.fasta.gz \\\n";
+        print F "  $techtype ./haplotype/haplotype-unknown.fasta.gz \\\n"     if ($withUnknown);
+        print F "> ./$asm-haplotype$haplotype.out 2>&1\n";
+        print F "\n";
+        print F "exit 0\n";
+        print F "\n";
+        close(F);
+
+        makeExecutable("./$asm-haplotype$haplotype.sh");
+    }
+
+    #  Fail if too many unassigned reads.
+
+    print STDERR "--\n";
+
+    if ($hapBases{"unknown"} / $totBases > 0.50) {
+        caExit("too many unassigned reads", undef);
+    }
+
+    #  Then run the scripts.
+
+    foreach my $haplotype (@haplotypes) {
+        if (getGlobal("useGrid") ne "1") {
+            print STDERR "-- Starting haplotype assembly $haplotype with script '$rootdir/$asm-haplotype$haplotype.sh'.\n";
+        } else {
+            print STDERR "-- Submitting '$rootdir/$asm-haplotype$haplotype.sh' to generate haplotype assembly $haplotype.\n";
+        }
+
+        runCommand(".", "./$asm-haplotype$haplotype.sh");
+    }
+
+    if (getGlobal("useGrid") ne "1") {
+        print STDERR "--\n";
+        print STDERR "-- Haplotype assemblies completed (or failed).\n";
+    } else {
+        print STDERR "--\n";
+        print STDERR "-- Haplotype assemblies submitted.\n";
+    }
+
+    exit(0);
+}
 
 if (setOptions($mode, "correct") eq "correct") {
-    if (sequenceFileExists("$asm.correctedReads") eq undef) {
+    if ((getNumberOfBasesInStore($asm, "obt") == 0) &&
+        (! fileExists("$asm.correctedReads.fasta.gz")) &&
+        (! fileExists("$asm.correctedReads.fastq.gz"))) {
+
+        submitScript($asm, undef);   #  See comments there as to why this is safe.
+
         print STDERR "--\n";
         print STDERR "--\n";
         print STDERR "-- BEGIN CORRECTION\n";
         print STDERR "--\n";
 
-        gatekeeper($asm, "cor", @inputFiles);
+        if (checkSequenceStore($asm, "cor", @inputFiles)) {
+            merylConfigure($asm, "cor");
+            merylCountCheck($asm, "cor")          foreach (1..getGlobal("canuIterationMax") + 1);
+            merylProcessCheck($asm, "cor")        foreach (1..getGlobal("canuIterationMax") + 1);
 
-        merylConfigure($asm, "cor");
-        merylCheck($asm, "cor")  foreach (1..getGlobal("canuIterationMax") + 1);
-        merylProcess($asm, "cor");
+            overlap($asm, "cor");
 
-        overlap($asm, "cor");
+            setupCorrectionParameters($asm);
 
-        buildCorrectionLayouts($asm);
-        generateCorrectedReads($asm)  foreach (1..getGlobal("canuIterationMax") + 1);
-        dumpCorrectedReads($asm);
+            buildCorrectionLayoutsConfigure($asm);
+            buildCorrectionLayoutsCheck($asm)     foreach (1..getGlobal("canuIterationMax") + 1);
 
-        buildHTML($asm, "cor");
+            filterCorrectionLayouts($asm);
+
+            generateCorrectedReadsConfigure($asm);
+            generateCorrectedReadsCheck($asm)     foreach (1..getGlobal("canuIterationMax") + 1);
+
+            loadCorrectedReads($asm);
+        }
     }
-
-    my $correctedReads = sequenceFileExists("$asm.correctedReads");
-
-    caExit("can't find corrected reads '$asm.correctedReads*' in directory '" . getcwd() . "'", undef)  if (!defined($correctedReads));
-
-    undef @inputFiles;
-    push  @inputFiles, "-$type-corrected\0$correctedReads";
 }
 
+dumpCorrectedReads($asm);
+
+if ((setOptions($mode, "trim") eq "trim") &&
+    (getGlobal("unitigger") ne "wtdbg")) {
+    if ((getNumberOfBasesInStore($asm, "utg") == 0) &&
+        (! fileExists("$asm.trimmedReads.fasta.gz")) &&
+        (! fileExists("$asm.trimmedReads.fastq.gz"))) {
+
+        submitScript($asm, undef);   #  See comments there as to why this is safe.
 
-if (setOptions($mode, "trim") eq "trim") {
-    if (sequenceFileExists("$asm.trimmedReads") eq undef) {
         print STDERR "--\n";
         print STDERR "--\n";
         print STDERR "-- BEGIN TRIMMING\n";
         print STDERR "--\n";
 
-        gatekeeper($asm, "obt", @inputFiles);
+        if (checkSequenceStore($asm, "obt", @inputFiles)) {
+            merylConfigure($asm, "obt");
+            merylCountCheck($asm, "obt")     foreach (1..getGlobal("canuIterationMax") + 1);
+            merylProcessCheck($asm, "obt")   foreach (1..getGlobal("canuIterationMax") + 1);
 
-        merylConfigure($asm, "obt");
-        merylCheck($asm, "obt")  foreach (1..getGlobal("canuIterationMax") + 1);
-        merylProcess($asm, "obt");
+            overlap($asm, "obt");
 
-        overlap($asm, "obt");
+            trimReads($asm);
+            splitReads($asm);
 
-        trimReads ($asm);
-        splitReads($asm);
-        dumpReads ($asm);
-        #summarizeReads($asm);
-
-        buildHTML($asm, "obt");
+            loadTrimmedReads($asm);
+        }
     }
-
-    my $trimmedReads = sequenceFileExists("$asm.trimmedReads");
-
-    caExit("can't find trimmed reads '$asm.trimmedReads*' in directory '" . getcwd() . "'", undef)  if (!defined($trimmedReads));
-
-    undef @inputFiles;
-    push  @inputFiles, "-$type-corrected\0$trimmedReads";
 }
 
+dumpTrimmedReads ($asm);
 
 if (setOptions($mode, "assemble") eq "assemble") {
-    if (sequenceFileExists("$asm.contigs") eq undef) {
+    if ((! fileExists("$asm.contigs.fasta")) &&
+        (! fileExists("$asm.contigs.fastq"))) {
+
+        submitScript($asm, undef);   #  See comments there as to why this is safe.
+
         print STDERR "--\n";
         print STDERR "--\n";
         print STDERR "-- BEGIN ASSEMBLY\n";
         print STDERR "--\n";
 
-        gatekeeper($asm, "utg", @inputFiles);
+        if (checkSequenceStore($asm, "utg", @inputFiles)) {
+            if (getGlobal("unitigger") ne "wtdbg") {
+                merylConfigure($asm, "utg");
+                merylCountCheck($asm, "utg")       foreach (1..getGlobal("canuIterationMax") + 1);
+                merylProcessCheck($asm, "utg")     foreach (1..getGlobal("canuIterationMax") + 1);
 
-        merylConfigure($asm, "utg");
-        merylCheck($asm, "utg")  foreach (1..getGlobal("canuIterationMax") + 1);
-        merylProcess($asm, "utg");
+                overlap($asm, "utg");
 
-        overlap($asm, "utg");
+                #readErrorDetection($asm);
 
-        #readErrorDetection($asm);
+                readErrorDetectionConfigure($asm);
+                readErrorDetectionCheck($asm)      foreach (1..getGlobal("canuIterationMax") + 1);
 
-        readErrorDetectionConfigure($asm);
-        readErrorDetectionCheck($asm)  foreach (1..getGlobal("canuIterationMax") + 1);
+                overlapErrorAdjustmentConfigure($asm);
+                overlapErrorAdjustmentCheck($asm)  foreach (1..getGlobal("canuIterationMax") + 1);
 
-        overlapErrorAdjustmentConfigure($asm);
-        overlapErrorAdjustmentCheck($asm)  foreach (1..getGlobal("canuIterationMax") + 1);
+                updateOverlapStore($asm);
+            }
 
-        updateOverlapStore($asm);
+            unitig($asm);
+            unitigCheck($asm)  foreach (1..getGlobal("canuIterationMax") + 1);
 
-        unitig($asm);
-        unitigCheck($asm)  foreach (1..getGlobal("canuIterationMax") + 1);
+            foreach (1..getGlobal("canuIterationMax") + 1) {   #  Consensus wants to change the script between the first and
+                consensusConfigure($asm);                      #  second iterations.  The script is rewritten in
+                consensusCheck($asm);                          #  consensusConfigure(), so we need to add that to the loop.
+            }
 
-        foreach (1..getGlobal("canuIterationMax") + 1) {   #  Consensus wants to change the script between the first and
-            consensusConfigure($asm);                      #  second iterations.  The script is rewritten in
-            consensusCheck($asm);                          #  consensusConfigure(), so we need to add that to the loop.
+            consensusLoad($asm);
+            consensusAnalyze($asm);
+
+            if (getGlobal("unitigger") ne "wtdbg") {
+                alignGFA($asm)  foreach (1..getGlobal("canuIterationMax") + 1);
+            }
+            generateOutputs($asm);
         }
-
-        consensusLoad($asm);
-        consensusAnalyze($asm);
-
-        alignGFA($asm)  foreach (1..getGlobal("canuIterationMax") + 1);
-
-        generateOutputs($asm);
     }
 }
 
