@@ -32,9 +32,11 @@ package canu::Configure;
 require Exporter;
 
 @ISA    = qw(Exporter);
-@EXPORT = qw(displayMemoryValue displayGenomeSize configureAssembler);
+@EXPORT = qw(getAllowedResources displayMemoryValue displayGenomeSize configureAssembler);
 
 use strict;
+use warnings "all";
+no  warnings "uninitialized";
 use Carp qw(cluck);
 use Sys::Hostname;
 
@@ -181,11 +183,14 @@ sub findGridMaxMemoryAndThreads () {
 #  the rest of canu - in particular, the part that runs the jobs - use the correct value.  Without
 #  resetting, I'd be making code changes all over the place to support the values returned.
 
-sub getAllowedResources ($$$$@) {
+sub getAllowedResources ($$$$$@);  #  Recursive call to getAllowedResources() wants the prototype.
+
+sub getAllowedResources ($$$$$@) {
     my $tag  = shift @_;  #  Variant, e.g., "cor", "utg"
     my $alg  = shift @_;  #  Algorithm, e.g., "mhap", "ovl"
     my $err  = shift @_;  #  Report of things we can't run.
     my $all  = shift @_;  #  Report of things we can run.
+    my $uni  = shift @_;  #  There's only one task to run (bogart, gfa)
     my $dbg  = shift @_;  #  Optional, report debugging stuff
 
     #  If no grid, or grid not enabled, everything falls under 'lcoal'.
@@ -286,7 +291,12 @@ sub getAllowedResources ($$$$@) {
     #    taskThreads = 4,8,32,64
     #    taskMemory  = 16g,32g,64g
 
-    my ($bestCores, $bestCoresM, $bestCoresT, $availMemoryMin, $availMemoryMax)  = (0, undef, undef, undef, undef);
+    my $bestCores      = 0;
+    my $bestMemory     = 16 * 1024 * 1024;   #  16 petabytes.
+    my $bestCoresM     = undef;
+    my $bestCoresT     = undef;
+    my $availMemoryMin = undef;
+    my $availMemoryMax = undef;
 
     foreach my $m (@taskMemory) {
         foreach my $t (@taskThreads) {
@@ -312,17 +322,19 @@ sub getAllowedResources ($$$$@) {
 
             for (my $ii=0; $ii<scalar(@gridCor); $ii++) {
                 my $np_cpu = $gridNum[$ii] * int($gridCor[$ii] / $t);  #  Each process uses $t cores, node has $gridCor[$ii] cores available.
-                my $np_mem = $gridNum[$ii] * int($gridMem[$ii] / $m);  #  Same idea.
+                my $np_mem = $gridNum[$ii] * int($gridMem[$ii] / $m);  #  Each process uses $m GBs,   node ame $gridMem[$ii] GBs   available.
 
-                my $np = ($np_cpu < $np_mem) ? $np_cpu : $np_mem;
+                my $np = ($np_cpu < $np_mem) ? $np_cpu : $np_mem;      #  Number of processes we can fit on this machine.
+
+                $np = 1  if ($uni);                                    #  But don't care if there is only one process to run!
 
                 if ($dbg) {
                     print STDERR "-- ERROR  for $t threads and $m memory - class$ii can support $np_cpu jobs(cores) and $np_mem jobs(memory), so $np jobs.\n";
                 }
 
-                $processes += $np;
-                $cores     += $np * $t;
-                $memory    += $np * $m;
+                $processes += $np;        #  Total number of processes running
+                $cores     += $np * $t;   #  Total cores in use
+                $memory    += $np * $m;   #  Total memory in use
             }
 
             if ($dbg) {
@@ -333,18 +345,19 @@ sub getAllowedResources ($$$$@) {
 
             next if ($cores == 0);
 
-            #  Save the best one seen so far.
+            #  Save the best one seen so far.  Break ties by selecting the one with the most memory.
 
-            if ($bestCores <= $cores) {
+            if (($bestCores <  $cores) ||
+                ($bestCores <= $cores) && ($bestMemory > $memory)) {
                 $bestCores  = $cores;
-                $bestCoresM = $m;
                 $bestCoresT = $t;
+                $bestCoresM = $m;
             }
         }
     }
 
     if (!defined($bestCoresM)) {
-        getAllowedResources($tag, $alg, $err, $all, 1)  if (!defined($dbg));
+        getAllowedResources($tag, $alg, $err, $all, $uni, 1)  if (!defined($dbg));
 
         print STDERR "-- ERROR\n";
         print STDERR "-- ERROR  Task $tag$alg can't run on any available machines.\n";
@@ -365,42 +378,53 @@ sub getAllowedResources ($$$$@) {
     #  Reset the global values for later use.  SPECIAL CASE!  For ovsMemory, we just want the list
     #  of valid memory sizes.
 
-    if ("$alg" ne "ovs") {
-        $taskMemory  = $bestCoresM;
-        $taskThreads = $bestCoresT;
-
-        setGlobal("${tag}${alg}Memory",  $taskMemory);
-        setGlobal("${tag}${alg}Threads", $taskThreads);
-
-    } else {
+    if ("$alg" eq "ovs") {
         $taskMemory  = $availMemoryMax;
         $taskThreads = $bestCoresT;
 
         setGlobal("${tag}${alg}Memory",  "$availMemoryMin-$availMemoryMax");
         setGlobal("${tag}${alg}Threads",  $taskThreads);
+
+    } else {
+        $taskMemory  = $bestCoresM;
+        $taskThreads = $bestCoresT;
+
+        setGlobal("${tag}${alg}Memory",  $taskMemory);
+        setGlobal("${tag}${alg}Threads", $taskThreads);
     }
 
     #  Check for stupidity.
 
-    caExit("invalid taskMemory=$taskMemory; maxMemory=$maxMemory", undef)     if ($taskMemory > $maxMemory);
+    caExit("invalid taskMemory=$taskMemory; maxMemory=$maxMemory", undef)     if ($taskMemory  > $maxMemory);
     caExit("invalid taskThread=$taskThreads; maxThreads=$maxThreads", undef)  if ($taskThreads > $maxThreads);
 
     #  Finally, reset the concurrency (if we're running locally) so we don't swamp our poor workstation.
 
-    my $concurrent = undef;
+    my $concurrent = undef;  #  Undef if in grid mode.
 
     if ($class eq "local") {
-        my $nc = int($maxThreads / $taskThreads);
+        my $nct = int($maxThreads / $taskThreads);
+        my $ncm = int($maxMemory  / $taskMemory);
 
-        if (($taskThreads * getGlobal("${tag}${alg}Concurrency") > $maxThreads)) {
-            $err .= "-- Reset concurrency from ", getGlobal("${tag}${alg}Concurrency"), " to $nc.\n";
+        my $nc  = ($nct < $ncm) ? $nct : $ncm;
+
+        $nc = 1  if ($uni);
+
+        #  If already set (on the command line), reset if too big.
+
+        if ($nc < getGlobal("${tag}${alg}Concurrency")) {
+            $err .= "-- Reset concurrency from " . getGlobal("${tag}${alg}Concurrency") . " to $nc.\n";
             setGlobal("${tag}${alg}Concurrency", $nc);
         }
 
+        #  If not set, set it.
+
         if (!defined(getGlobal("${tag}${alg}Concurrency"))) {
             setGlobal("${tag}${alg}Concurrency", $nc);
         }
 
+        #  Update the local variable for the report.
+
         $concurrent = getGlobal("${tag}${alg}Concurrency");
     }
 
@@ -408,25 +432,30 @@ sub getAllowedResources ($$$$@) {
 
     my $nam;
 
-    if    ($alg eq "meryl")    {  $nam = "(k-mer counting)"; }
-    elsif ($alg eq "mhap")     {  $nam = "(overlap detection with mhap)"; }
-    elsif ($alg eq "mmap")     {  $nam = "(overlap detection with minimap)"; }
-    elsif ($alg eq "ovl")      {  $nam = "(overlap detection)"; }
-    elsif ($alg eq "cor")      {  $nam = "(read correction)"; }
-    elsif ($alg eq "ovb")      {  $nam = "(overlap store bucketizer)"; }
-    elsif ($alg eq "ovs")      {  $nam = "(overlap store sorting)"; }
-    elsif ($alg eq "red")      {  $nam = "(read error detection)"; }
-    elsif ($alg eq "oea")      {  $nam = "(overlap error adjustment)"; }
-    elsif ($alg eq "bat")      {  $nam = "(contig construction)"; }
-    elsif ($alg eq "cns")      {  $nam = "(consensus)"; }
-    elsif ($alg eq "gfa")      {  $nam = "(GFA alignment and processing)"; }
+    if    ($alg eq "meryl")     {  $nam = "(k-mer counting)"; }
+    elsif ($alg eq "hap")       {  $nam = "(read-to-haplotype assignment)"; }
+    elsif ($alg eq "mhap")      {  $nam = "(overlap detection with mhap)"; }
+    elsif ($alg eq "mmap")      {  $nam = "(overlap detection with minimap)"; }
+    elsif ($alg eq "ovl")       {  $nam = "(overlap detection)"; }
+    elsif ($alg eq "cor")       {  $nam = "(read correction)"; }
+    elsif ($alg eq "ovb")       {  $nam = "(overlap store bucketizer)"; }
+    elsif ($alg eq "ovs")       {  $nam = "(overlap store sorting)"; }
+    elsif ($alg eq "red")       {  $nam = "(read error detection)"; }
+    elsif ($alg eq "oea")       {  $nam = "(overlap error adjustment)"; }
+    elsif ($alg eq "bat")       {  $nam = "(contig construction with bogart)"; }
+    elsif ($alg eq "dbg")       {  $nam = "(contig construction with wtdbg)"; }
+    elsif ($alg eq "cns")       {  $nam = "(consensus)"; }
+    elsif ($alg eq "gfa")       {  $nam = "(GFA alignment and processing)"; }
     else {
         caFailure("unknown task '$alg' in getAllowedResources().", undef);
     }
 
-    my $job = substr("    $concurrent",  -3) . " job" . (($concurrent == 1) ? " " : "s");
-    my $thr = substr("    $taskThreads", -3) . " CPU" . (($taskThreads == 1) ? " " : "s");
-    my $mem = substr("    $taskMemory",  -4) . " GB";
+    my $mem  = substr("    $taskMemory",  -4) . " GB";
+    my $thr  = substr("    $taskThreads", -3) . " CPU" . (($taskThreads == 1) ? " " : "s");
+    my $job  = substr("    $concurrent",  -3) . " job" . (($concurrent == 1) ? " " : "s");
+
+    my $memt = substr("     " . $taskMemory  * $concurrent, -4) . " GB";
+    my $thrt = substr("     " . $taskThreads * $concurrent, -4) . " CPU" . (($taskThreads * $concurrent == 1) ? " " : "s");
 
     my $t = substr("$tag$alg     ", 0, 7);
 
@@ -438,8 +467,8 @@ sub getAllowedResources ($$$$@) {
             $all .= "--                            (tag)Concurrency\n";
             $all .= "--                     (tag)Threads          |\n";
             $all .= "--            (tag)Memory         |          |\n";
-            $all .= "--        (tag)         |         |          |  algorithm\n";
-            $all .= "--        -------  ------  --------   --------  -----------------------------\n";
+            $all .= "--        (tag)         |         |          |     total usage     algorithm\n";
+            $all .= "--        -------  ------  --------   --------  -----------------  -----------------------------\n";
         } else {
             $all .= "--                     (tag)Threads\n";
             $all .= "--            (tag)Memory         |\n";
@@ -447,8 +476,8 @@ sub getAllowedResources ($$$$@) {
             $all .= "--        -------  ------  --------  -----------------------------\n";
         }
     }
-    $all .= "-- Local: $t $mem  $thr x $job  $nam\n"      if ( defined($concurrent));
-    $all .= "-- Grid:  $t $mem  $thr  $nam\n"             if (!defined($concurrent));
+    $all .= "-- Local: $t $mem  $thr x $job  $memt $thrt  $nam\n"      if ( defined($concurrent));
+    $all .= "-- Grid:  $t $mem  $thr  $nam\n"                          if (!defined($concurrent));
 
     return($err, $all);
 }
@@ -525,10 +554,11 @@ sub configureAssembler () {
 
     setGlobal("genomeSize", adjustGenomeSize(getGlobal("genomeSize")));
 
+    setGlobal("executiveMemory", adjustMemoryValue(getGlobal("executiveMemory")));
+
     setGlobal("minMemory",  adjustMemoryValue(getGlobal("minMemory")));
     setGlobal("maxMemory",  adjustMemoryValue(getGlobal("maxMemory")));
 
-
     #  For overlapper and mhap, allow larger maximums for larger genomes.  More memory won't help
     #  smaller genomes, and the smaller minimums won't hurt larger genomes (which are probably being
     #  run on larger machines anyway, so the minimums won't be used).
@@ -536,24 +566,63 @@ sub configureAssembler () {
     #  For uncorrected overlapper, both memory and thread count is reduced.  Memory because it is
     #  very CPU bound, and thread count because it can be quite unbalanced.
 
+    #  22 bits ->   864 MB table structure,   64 million kmers
+    #  23 bits ->  1728 MB table structure,  128 million kmers
+    #  24 bits ->  3456 MB table structure,  256 million kmers
+    #  25 bits ->  6912 MB table structure,  512 million kmers
+    #  26 bits -> 13824 MB table structure, 1024 million kmers
+    #
+    #    sequence generate -min 5000 -max 25000 -bases 10000000000                                   > random.fasta
+    #    sequence generate -min 5000 -max 25000 -bases 10000000000 -a 0.9 -c 0.033 -g 0.033 -t 0.033 > repeat.fasta
+    #
+    #               TABLE    W/DATA
+    #    bits 20   216 MB -  2500 MB random -   16 million kmers at 75% load
+    #    bits 21   432 MB -  2750 MB random -   32 million kmers
+    #
+    #    bits 22   864 MB -  3000 MB random -   64 million kmers
+    #
+    #    bits 23  1728 MB -  3500 MB random -  128 million kmers
+    #
+    #    bits 24  3456 MB -  5750 MB random -  200 million kmers at 56% load
+    #             3456 MB -  6500 MB random -  256 million kmers at 75% load
+    #             3456 MB -  8200 MB repeat -  256 million kmers at  5% load
+    #
+    #    bits 25  6912 MB - 10000 MB random -  300 million kmers at 42% load
+    #             6912 MB - 12750 MB random -  512 million kmers at 75% load
+    #             6912 MB - 21000 MB repeat -  600 million kmers at  5% load
+    #
+    #    bits 26 13824 MB - 25000 MB random - 1024 million kmers at 75% load
+    #
+    #
+    #  An expansion factor for the bases to load into the hash table.  Each table size will hold a
+    #  little more than the above number of different kmers.  The additional third in the expansion
+    #  is to adjust for repeated kmers in the reads.
+
+    my $hx = 1.25 * 1000000;
+
     if      (getGlobal("genomeSize") < adjustGenomeSize("40m")) {
-        setGlobalIfUndef("corOvlMemory", "2-6");     setGlobalIfUndef("corOvlThreads", "1");
-        setGlobalIfUndef("obtOvlMemory", "4-8");     setGlobalIfUndef("obtOvlThreads", "1-8");
-        setGlobalIfUndef("utgOvlMemory", "4-8");     setGlobalIfUndef("utgOvlThreads", "1-8");
+        setGlobalIfUndef("corOvlHashBlockLength",     2500000);    setGlobalIfUndef("obtOvlHashBlockLength",    64 * $hx);    setGlobalIfUndef("utgOvlHashBlockLength",    64 * $hx);
+        setGlobalIfUndef("corOvlRefBlockLength",      2000000);    setGlobalIfUndef("obtOvlRefBlockLength",   1000000000);    setGlobalIfUndef("utgOvlRefBlockLength",   1000000000);   #    1 Gbp
 
-        setGlobalIfUndef("corMhapMemory", "4-6");   setGlobalIfUndef("corMhapThreads", "1-16");
-        setGlobalIfUndef("obtMhapMemory", "4-6");   setGlobalIfUndef("obtMhapThreads", "1-16");
-        setGlobalIfUndef("utgMhapMemory", "4-6");   setGlobalIfUndef("utgMhapThreads", "1-16");
+        setGlobalIfUndef("corOvlMemory", "2");       setGlobalIfUndef("corOvlThreads", "1");      setGlobalIfUndef("corOvlHashBits", 22);
+        setGlobalIfUndef("obtOvlMemory", "4");       setGlobalIfUndef("obtOvlThreads", "2-8");    setGlobalIfUndef("obtOvlHashBits", 22);
+        setGlobalIfUndef("utgOvlMemory", "4");       setGlobalIfUndef("utgOvlThreads", "2-8");    setGlobalIfUndef("utgOvlHashBits", 22);
 
-        setGlobalIfUndef("corMMapMemory", "4-6");   setGlobalIfUndef("corMMapThreads", "1-16");
-        setGlobalIfUndef("obtMMapMemory", "4-6");   setGlobalIfUndef("obtMMapThreads", "1-16");
-        setGlobalIfUndef("utgMMapMemory", "4-6");   setGlobalIfUndef("utgMMapThreads", "1-16");
+        setGlobalIfUndef("corMhapMemory", "4-6");    setGlobalIfUndef("corMhapThreads", "1-16");
+        setGlobalIfUndef("obtMhapMemory", "4-6");    setGlobalIfUndef("obtMhapThreads", "1-16");
+        setGlobalIfUndef("utgMhapMemory", "4-6");    setGlobalIfUndef("utgMhapThreads", "1-16");
 
+        setGlobalIfUndef("corMMapMemory", "4-6");    setGlobalIfUndef("corMMapThreads", "1-16");
+        setGlobalIfUndef("obtMMapMemory", "4-6");    setGlobalIfUndef("obtMMapThreads", "1-16");
+        setGlobalIfUndef("utgMMapMemory", "4-6");    setGlobalIfUndef("utgMMapThreads", "1-16");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("500m")) {
-        setGlobalIfUndef("corOvlMemory", "2-6");     setGlobalIfUndef("corOvlThreads", "1");
-        setGlobalIfUndef("obtOvlMemory", "4-8");     setGlobalIfUndef("obtOvlThreads", "1-8");
-        setGlobalIfUndef("utgOvlMemory", "4-8");     setGlobalIfUndef("utgOvlThreads", "1-8");
+        setGlobalIfUndef("corOvlHashBlockLength",     2500000);    setGlobalIfUndef("obtOvlHashBlockLength",   128 * $hx);    setGlobalIfUndef("utgOvlHashBlockLength",   128 * $hx);
+        setGlobalIfUndef("corOvlRefBlockLength",      2000000);    setGlobalIfUndef("obtOvlRefBlockLength",   5000000000);    setGlobalIfUndef("utgOvlRefBlockLength",   5000000000);   #    5 Gbp
+
+        setGlobalIfUndef("corOvlMemory", "2");       setGlobalIfUndef("corOvlThreads", "1");      setGlobalIfUndef("corOvlHashBits", 23);
+        setGlobalIfUndef("obtOvlMemory", "8");       setGlobalIfUndef("obtOvlThreads", "2-8");    setGlobalIfUndef("obtOvlHashBits", 23);
+        setGlobalIfUndef("utgOvlMemory", "8");       setGlobalIfUndef("utgOvlThreads", "2-8");    setGlobalIfUndef("utgOvlHashBits", 23);
 
         setGlobalIfUndef("corMhapMemory", "8-13");   setGlobalIfUndef("corMhapThreads", "1-16");
         setGlobalIfUndef("obtMhapMemory", "8-13");   setGlobalIfUndef("obtMhapThreads", "1-16");
@@ -564,22 +633,28 @@ sub configureAssembler () {
         setGlobalIfUndef("utgMMapMemory", "8-13");   setGlobalIfUndef("utgMMapThreads", "1-16");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("2g")) {
-        setGlobalIfUndef("corOvlMemory", "2-8");     setGlobalIfUndef("corOvlThreads", "1");
-        setGlobalIfUndef("obtOvlMemory", "4-12");    setGlobalIfUndef("obtOvlThreads", "1-8");
-        setGlobalIfUndef("utgOvlMemory", "4-12");    setGlobalIfUndef("utgOvlThreads", "1-8");
+        setGlobalIfUndef("corOvlHashBlockLength",     2500000);    setGlobalIfUndef("obtOvlHashBlockLength",   256 * $hx);    setGlobalIfUndef("utgOvlHashBlockLength",   256 * $hx);
+        setGlobalIfUndef("corOvlRefBlockLength",      2000000);    setGlobalIfUndef("obtOvlRefBlockLength",  15000000000);    setGlobalIfUndef("utgOvlRefBlockLength",  15000000000);   #   15 Gbp
 
-        setGlobalIfUndef("corMhapMemory", "16-32");   setGlobalIfUndef("corMhapThreads", "4-16");
-        setGlobalIfUndef("obtMhapMemory", "16-32");   setGlobalIfUndef("obtMhapThreads", "4-16");
-        setGlobalIfUndef("utgMhapMemory", "16-32");   setGlobalIfUndef("utgMhapThreads", "4-16");
+        setGlobalIfUndef("corOvlMemory", "8");       setGlobalIfUndef("corOvlThreads", "1");      setGlobalIfUndef("corOvlHashBits", 24);
+        setGlobalIfUndef("obtOvlMemory", "16");      setGlobalIfUndef("obtOvlThreads", "4-16");   setGlobalIfUndef("obtOvlHashBits", 24);
+        setGlobalIfUndef("utgOvlMemory", "16");      setGlobalIfUndef("utgOvlThreads", "4-16");   setGlobalIfUndef("utgOvlHashBits", 24);
 
-        setGlobalIfUndef("corMMapMemory", "16-32");   setGlobalIfUndef("corMMapThreads", "1-16");
-        setGlobalIfUndef("obtMMapMemory", "16-32");   setGlobalIfUndef("obtMMapThreads", "1-16");
-        setGlobalIfUndef("utgMMapMemory", "16-32");   setGlobalIfUndef("utgMMapThreads", "1-16");
+        setGlobalIfUndef("corMhapMemory", "16-32");  setGlobalIfUndef("corMhapThreads", "4-16");
+        setGlobalIfUndef("obtMhapMemory", "16-32");  setGlobalIfUndef("obtMhapThreads", "4-16");
+        setGlobalIfUndef("utgMhapMemory", "16-32");  setGlobalIfUndef("utgMhapThreads", "4-16");
+
+        setGlobalIfUndef("corMMapMemory", "16-32");  setGlobalIfUndef("corMMapThreads", "1-16");
+        setGlobalIfUndef("obtMMapMemory", "16-32");  setGlobalIfUndef("obtMMapThreads", "1-16");
+        setGlobalIfUndef("utgMMapMemory", "16-32");  setGlobalIfUndef("utgMMapThreads", "1-16");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("5g")) {
-        setGlobalIfUndef("corOvlMemory", "2-8");     setGlobalIfUndef("corOvlThreads", "1");
-        setGlobalIfUndef("obtOvlMemory", "4-16");    setGlobalIfUndef("obtOvlThreads", "1-8");
-        setGlobalIfUndef("utgOvlMemory", "4-16");    setGlobalIfUndef("utgOvlThreads", "1-8");
+        setGlobalIfUndef("corOvlHashBlockLength",     2500000);    setGlobalIfUndef("obtOvlHashBlockLength",   512 * $hx);    setGlobalIfUndef("utgOvlHashBlockLength",   512 * $hx);
+        setGlobalIfUndef("corOvlRefBlockLength",      2000000);    setGlobalIfUndef("obtOvlRefBlockLength",  20000000000);    setGlobalIfUndef("utgOvlRefBlockLength",  20000000000);   #   20 Gbp
+
+        setGlobalIfUndef("corOvlMemory", "8");       setGlobalIfUndef("corOvlThreads", "1");      setGlobalIfUndef("corOvlHashBits", 25);
+        setGlobalIfUndef("obtOvlMemory", "24");      setGlobalIfUndef("obtOvlThreads", "4-16");   setGlobalIfUndef("obtOvlHashBits", 25);
+        setGlobalIfUndef("utgOvlMemory", "24");      setGlobalIfUndef("utgOvlThreads", "4-16");   setGlobalIfUndef("utgOvlHashBits", 25);
 
         setGlobalIfUndef("corMhapMemory", "16-48");  setGlobalIfUndef("corMhapThreads", "4-16");
         setGlobalIfUndef("obtMhapMemory", "16-48");  setGlobalIfUndef("obtMhapThreads", "4-16");
@@ -590,9 +665,12 @@ sub configureAssembler () {
         setGlobalIfUndef("utgMMapMemory", "16-48");  setGlobalIfUndef("utgMMapThreads", "1-16");
 
     } else {
-        setGlobalIfUndef("corOvlMemory", "2-8");     setGlobalIfUndef("corOvlThreads", "1");
-        setGlobalIfUndef("obtOvlMemory", "4-16");    setGlobalIfUndef("obtOvlThreads", "1-8");
-        setGlobalIfUndef("utgOvlMemory", "4-16");    setGlobalIfUndef("utgOvlThreads", "1-8");
+        setGlobalIfUndef("corOvlHashBlockLength",     2500000);    setGlobalIfUndef("obtOvlHashBlockLength",   512 * $hx);    setGlobalIfUndef("utgOvlHashBlockLength",   512 * $hx);
+        setGlobalIfUndef("corOvlRefBlockLength",      2000000);    setGlobalIfUndef("obtOvlRefBlockLength",  30000000000);    setGlobalIfUndef("utgOvlRefBlockLength",  30000000000);   #   30 Gbp
+
+        setGlobalIfUndef("corOvlMemory", "8");       setGlobalIfUndef("corOvlThreads", "1");      setGlobalIfUndef("corOvlHashBits", 25);
+        setGlobalIfUndef("obtOvlMemory", "24");      setGlobalIfUndef("obtOvlThreads", "4-16");   setGlobalIfUndef("obtOvlHashBits", 25);
+        setGlobalIfUndef("utgOvlMemory", "24");      setGlobalIfUndef("utgOvlThreads", "4-16");   setGlobalIfUndef("utgOvlHashBits", 25);
 
         setGlobalIfUndef("corMhapMemory", "32-64");  setGlobalIfUndef("corMhapThreads", "4-16");
         setGlobalIfUndef("obtMhapMemory", "32-64");  setGlobalIfUndef("obtMhapThreads", "4-16");
@@ -603,64 +681,81 @@ sub configureAssembler () {
         setGlobalIfUndef("utgMMapMemory", "32-64");  setGlobalIfUndef("utgMMapThreads", "1-16");
     }
 
-    #  Overlapper block sizes probably don't need to be modified based on genome size.
-
-    setGlobalIfUndef("corOvlHashBlockLength",   2500000);   setGlobalIfUndef("corOvlRefBlockSize",   20000);   setGlobalIfUndef("corOvlRefBlockLength", 0);
-    setGlobalIfUndef("obtOvlHashBlockLength", 100000000);   setGlobalIfUndef("obtOvlRefBlockSize", 2000000);   setGlobalIfUndef("obtOvlRefBlockLength", 0);
-    setGlobalIfUndef("utgOvlHashBlockLength", 100000000);   setGlobalIfUndef("utgOvlRefBlockSize", 2000000);   setGlobalIfUndef("utgOvlRefBlockLength", 0);
-
     #  Overlap store construction should be based on the number of overlaps, but we obviously don't
     #  know that until much later.  If we set memory too large, we risk (in the parallel version for sure)
     #  inefficiency; too small and we run out of file handles.
 
     if      (getGlobal("genomeSize") < adjustGenomeSize("300m")) {
-        setGlobalIfUndef("ovsMethod", "sequential");
-        setGlobalIfUndef("ovbMemory",   "2-4");     setGlobalIfUndef("ovbThreads",   "1");
-        setGlobalIfUndef("ovsMemory",   "2-8");     setGlobalIfUndef("ovsThreads",   "1");
+        setGlobalIfUndef("ovbMemory",   "4");       setGlobalIfUndef("ovbThreads",   "1");
+        setGlobalIfUndef("ovsMemory",   "4-8");     setGlobalIfUndef("ovsThreads",   "1");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("1g")) {
-        setGlobalIfUndef("ovsMethod", "parallel");
-        setGlobalIfUndef("ovbMemory",   "2-4");     setGlobalIfUndef("ovbThreads",   "1");
-        setGlobalIfUndef("ovsMemory",   "4-16");    setGlobalIfUndef("ovsThreads",   "1");
+        setGlobalIfUndef("ovbMemory",   "4");       setGlobalIfUndef("ovbThreads",   "1");
+        setGlobalIfUndef("ovsMemory",   "8-16");    setGlobalIfUndef("ovsThreads",   "1");
 
     } else {
-        setGlobalIfUndef("ovsMethod", "parallel");
-        setGlobalIfUndef("ovbMemory",   "2-4");     setGlobalIfUndef("ovbThreads",   "1");
-        setGlobalIfUndef("ovsMemory",   "4-32");    setGlobalIfUndef("ovsThreads",   "1");
+        setGlobalIfUndef("ovbMemory",   "4");       setGlobalIfUndef("ovbThreads",   "1");
+        setGlobalIfUndef("ovsMemory",   "16-32");   setGlobalIfUndef("ovsThreads",   "1");
     }
 
     #  Correction and consensus are somewhat invariant.
+    #    Correction memory is set based on read length in CorrectReads.pm.
+    #    Consensus memory is set based on tig size in Consensus.pm.
 
     if      (getGlobal("genomeSize") < adjustGenomeSize("40m")) {
-        setGlobalIfUndef("cnsMemory",     "8-32");     setGlobalIfUndef("cnsThreads",      "1-4");
-        setGlobalIfUndef("corMemory",     "6-16");     setGlobalIfUndef("corThreads",      "1-2");
+        setGlobalIfUndef("cnsMemory",     undef);      setGlobalIfUndef("cnsThreads",      "1-4");
+        setGlobalIfUndef("corMemory",     undef);      setGlobalIfUndef("corThreads",      "4");
         setGlobalIfUndef("cnsPartitions", "8");        setGlobalIfUndef("cnsPartitionMin", "15000");
         setGlobalIfUndef("corPartitions", "256");      setGlobalIfUndef("corPartitionMin", "5000");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("1g")) {
-        setGlobalIfUndef("cnsMemory",     "16-48");    setGlobalIfUndef("cnsThreads",      "2-8");
-        setGlobalIfUndef("corMemory",     "6-20");     setGlobalIfUndef("corThreads",      "2-4");
+        setGlobalIfUndef("cnsMemory",     undef);      setGlobalIfUndef("cnsThreads",      "2-8");
+        setGlobalIfUndef("corMemory",     undef);      setGlobalIfUndef("corThreads",      "4");
         setGlobalIfUndef("cnsPartitions", "64");       setGlobalIfUndef("cnsPartitionMin", "20000");
         setGlobalIfUndef("corPartitions", "512");      setGlobalIfUndef("corPartitionMin", "10000");
 
     } else {
-        setGlobalIfUndef("cnsMemory",     "64-128");   setGlobalIfUndef("cnsThreads",      "2-8");
-        setGlobalIfUndef("corMemory",     "10-32");    setGlobalIfUndef("corThreads",      "2-4");
+        setGlobalIfUndef("cnsMemory",     undef);      setGlobalIfUndef("cnsThreads",      "2-8");
+        setGlobalIfUndef("corMemory",     undef);      setGlobalIfUndef("corThreads",      "4");
         setGlobalIfUndef("cnsPartitions", "256");      setGlobalIfUndef("cnsPartitionMin", "25000");
         setGlobalIfUndef("corPartitions", "1024");     setGlobalIfUndef("corPartitionMin", "15000");
     }
 
     #  Meryl too, basically just small or big.  This should really be using the number of bases
-    #  reported from gatekeeper.
+    #  reported from sqStore.
 
     if      (getGlobal("genomeSize") < adjustGenomeSize("100m")) {
-        setGlobalIfUndef("merylMemory", "4-8");     setGlobalIfUndef("merylThreads", "1-4");
+        setGlobalIfUndef("merylMemory", "4-12");        setGlobalIfUndef("merylThreads", "1-4");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("1g")) {
-        setGlobalIfUndef("merylMemory", "16-64");    setGlobalIfUndef("merylThreads", "1-16");
+        setGlobalIfUndef("merylMemory", "12-24");       setGlobalIfUndef("merylThreads", "1-8");
 
     } else {
-        setGlobalIfUndef("merylMemory", "64-256");   setGlobalIfUndef("merylThreads", "1-32");
+        setGlobalIfUndef("merylMemory", "24-64");       setGlobalIfUndef("merylThreads", "1-8");
+    }
+
+    #  Total guesses on read-to-haplotype assignment.  A well-behaved nanopore human
+    #  was running in 2GB memory.
+
+    if      (getGlobal("genomeSize") < adjustGenomeSize("100m")) {
+        setGlobalIfUndef("hapMemory", "4-8");     setGlobalIfUndef("hapThreads", "1-4");
+
+    } elsif (getGlobal("genomeSize") < adjustGenomeSize("1g")) {
+        setGlobalIfUndef("hapMemory", "6-12");    setGlobalIfUndef("hapThreads", "8-24");
+
+    } else {
+        setGlobalIfUndef("hapMemory", "8-16");    setGlobalIfUndef("hapThreads", "16-64");
+    }
+
+
+    if (getGlobal("genomeSize") < 10000000) {
+        setGlobal("corOvlMerDistinct",  "0.9999")   if (!defined(getGlobal("corOvlMerThreshold")) && !defined(getGlobal("corOvlMerDistinct")));
+        setGlobal("obtOvlMerDistinct",  "0.9999")   if (!defined(getGlobal("obtOvlMerThreshold")) && !defined(getGlobal("obtOvlMerDistinct")));
+        setGlobal("utgOvlMerDistinct",  "0.9999")   if (!defined(getGlobal("utgOvlMerThreshold")) && !defined(getGlobal("utgOvlMerDistinct")));
+    } else {
+        setGlobal("corOvlMerDistinct",  "0.9990")   if (!defined(getGlobal("corOvlMerThreshold")) && !defined(getGlobal("corOvlMerDistinct")));
+        setGlobal("obtOvlMerDistinct",  "0.9990")   if (!defined(getGlobal("obtOvlMerThreshold")) && !defined(getGlobal("obtOvlMerDistinct")));
+        setGlobal("utgOvlMerDistinct",  "0.9990")   if (!defined(getGlobal("utgOvlMerThreshold")) && !defined(getGlobal("utgOvlMerDistinct")));
     }
 
     #  Overlap error adjustment
@@ -675,31 +770,31 @@ sub configureAssembler () {
     #   On drosophila, with 270,000 reads (median len ~17,000bp), this will result in about  50 jobs.
     #   The memory-only limit generated 36 jobs.
     #
-    setGlobalIfUndef("redBatchSize",   "5000");
-    setGlobalIfUndef("redBatchLength", "");
+    setGlobalIfUndef("redBatchSize",   undef);
+    setGlobalIfUndef("redBatchLength", "500000000");
 
-    setGlobalIfUndef("oeaBatchSize",   "25000");
-    setGlobalIfUndef("oeaBatchLength", "");
+    setGlobalIfUndef("oeaBatchSize",   undef);
+    setGlobalIfUndef("oeaBatchLength", "300000000");
 
     if      (getGlobal("genomeSize") < adjustGenomeSize("40m")) {
-        setGlobalIfUndef("redMemory",   "1-2");    setGlobalIfUndef("redThreads",   "1-4");
-        setGlobalIfUndef("oeaMemory",   "1");      setGlobalIfUndef("oeaThreads",   "1");
+        setGlobalIfUndef("redMemory", "4-8");         setGlobalIfUndef("redThreads", "2-4");
+        setGlobalIfUndef("oeaMemory", "4");           setGlobalIfUndef("oeaThreads", "1");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("500m")) {
-        setGlobalIfUndef("redMemory",   "2-6");    setGlobalIfUndef("redThreads",   "1-6");
-        setGlobalIfUndef("oeaMemory",   "2");       setGlobalIfUndef("oeaThreads",   "1");
+        setGlobalIfUndef("redMemory", "6-10");        setGlobalIfUndef("redThreads", "4-6");
+        setGlobalIfUndef("oeaMemory", "4");           setGlobalIfUndef("oeaThreads", "1");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("2g")) {
-        setGlobalIfUndef("redMemory",   "2-8");    setGlobalIfUndef("redThreads",   "1-8");
-        setGlobalIfUndef("oeaMemory",   "2");       setGlobalIfUndef("oeaThreads",   "1");
+        setGlobalIfUndef("redMemory", "8-12");         setGlobalIfUndef("redThreads", "4-8");
+        setGlobalIfUndef("oeaMemory", "4");           setGlobalIfUndef("oeaThreads", "1");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("5g")) {
-        setGlobalIfUndef("redMemory",   "2-16");    setGlobalIfUndef("redThreads",   "1-8");
-        setGlobalIfUndef("oeaMemory",   "4");       setGlobalIfUndef("oeaThreads",   "1");
+        setGlobalIfUndef("redMemory", "8-16");        setGlobalIfUndef("redThreads", "4-8");
+        setGlobalIfUndef("oeaMemory", "8");           setGlobalIfUndef("oeaThreads", "1");
 
     } else {
-        setGlobalIfUndef("redMemory",   "2-16");    setGlobalIfUndef("redThreads",   "1-8");
-        setGlobalIfUndef("oeaMemory",   "4");       setGlobalIfUndef("oeaThreads",   "1");
+        setGlobalIfUndef("redMemory", "12-20");       setGlobalIfUndef("redThreads", "6-10");
+        setGlobalIfUndef("oeaMemory", "8");           setGlobalIfUndef("oeaThreads", "1");
     }
 
     #  And bogart and GFA alignment/processing.
@@ -707,24 +802,34 @@ sub configureAssembler () {
     #  GFA for genomes less than 40m is run in the canu process itself.
 
     if      (getGlobal("genomeSize") < adjustGenomeSize("40m")) {
-        setGlobalIfUndef("batMemory",   "2-16");        setGlobalIfUndef("batThreads",   "1-4");
-        setGlobalIfUndef("gfaMemory",   "2-8");         setGlobalIfUndef("gfaThreads",   "1-4");
+        setGlobalIfUndef("batMemory", "4-16");        setGlobalIfUndef("batThreads", "2-4");
+        setGlobalIfUndef("dbgMemory", "4-16");        setGlobalIfUndef("dbgThreads", "2-4");
+
+        setGlobalIfUndef("gfaMemory", "2-8");         setGlobalIfUndef("gfaThreads", "1-4");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("500m")) {
-        setGlobalIfUndef("batMemory",   "16-64");       setGlobalIfUndef("batThreads",   "2-8");
-        setGlobalIfUndef("gfaMemory",   "4-8");         setGlobalIfUndef("gfaThreads",   "2-8");
+        setGlobalIfUndef("batMemory", "16-64");       setGlobalIfUndef("batThreads", "2-8");
+        setGlobalIfUndef("dbgMemory", "8-64");       setGlobalIfUndef("dbgThreads", "2-8");
+
+        setGlobalIfUndef("gfaMemory", "4-8");         setGlobalIfUndef("gfaThreads", "2-8");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("2g")) {
-        setGlobalIfUndef("batMemory",   "32-256");      setGlobalIfUndef("batThreads",   "4-16");
-        setGlobalIfUndef("gfaMemory",   "8-16");         setGlobalIfUndef("gfaThreads",  "4-16");
+        setGlobalIfUndef("batMemory", "32-256");      setGlobalIfUndef("batThreads", "4-16");
+        setGlobalIfUndef("dbgMemory", "32-256");      setGlobalIfUndef("dbgThreads", "4-16");
+
+        setGlobalIfUndef("gfaMemory", "8-16");        setGlobalIfUndef("gfaThreads", "4-16");
 
     } elsif (getGlobal("genomeSize") < adjustGenomeSize("5g")) {
-        setGlobalIfUndef("batMemory",   "128-512");     setGlobalIfUndef("batThreads",   "8-32");
-        setGlobalIfUndef("gfaMemory",   "16-32");        setGlobalIfUndef("gfaThreads",  "8-32");
+        setGlobalIfUndef("batMemory", "128-512");     setGlobalIfUndef("batThreads", "8-32");
+        setGlobalIfUndef("dbgMemory", "128-512");     setGlobalIfUndef("dbgThreads", "8-32");
+
+        setGlobalIfUndef("gfaMemory", "16-32");       setGlobalIfUndef("gfaThreads", "8-32");
 
     } else {
-        setGlobalIfUndef("batMemory",   "256-1024");    setGlobalIfUndef("batThreads",   "16-64");
-        setGlobalIfUndef("gfaMemory",   "32-64");       setGlobalIfUndef("gfaThreads",   "16-64");
+        setGlobalIfUndef("batMemory", "256-1024");    setGlobalIfUndef("batThreads", "16-64");
+        setGlobalIfUndef("dbgMemory", "128-1024");    setGlobalIfUndef("dbgThreads", "16-64");
+
+        setGlobalIfUndef("gfaMemory", "32-64");       setGlobalIfUndef("gfaThreads", "16-64");
     }
 
 
@@ -739,33 +844,37 @@ sub configureAssembler () {
     my $err;
     my $all;
 
-    ($err, $all) = getAllowedResources("",    "meryl",    $err, $all);
+    ($err, $all) = getAllowedResources("",    "meryl",     $err, $all, 0);
 
-    ($err, $all) = getAllowedResources("cor", "mhap",     $err, $all)   if (getGlobal("corOverlapper") eq "mhap");
-    ($err, $all) = getAllowedResources("cor", "mmap",     $err, $all)   if (getGlobal("corOverlapper") eq "minimap");
-    ($err, $all) = getAllowedResources("cor", "ovl",      $err, $all)   if (getGlobal("corOverlapper") eq "ovl");
+    ($err, $all) = getAllowedResources("",    "hap",       $err, $all, 0);
 
-    ($err, $all) = getAllowedResources("obt", "mhap",     $err, $all)   if (getGlobal("obtOverlapper") eq "mhap");
-    ($err, $all) = getAllowedResources("obt", "mmap",     $err, $all)   if (getGlobal("obtOverlapper") eq "minimap");
-    ($err, $all) = getAllowedResources("obt", "ovl",      $err, $all)   if (getGlobal("obtOverlapper") eq "ovl");
+    ($err, $all) = getAllowedResources("cor", "mhap",      $err, $all, 0)   if (getGlobal("corOverlapper") eq "mhap");
+    ($err, $all) = getAllowedResources("cor", "mmap",      $err, $all, 0)   if (getGlobal("corOverlapper") eq "minimap");
+    ($err, $all) = getAllowedResources("cor", "ovl",       $err, $all, 0)   if (getGlobal("corOverlapper") eq "ovl");
 
-    ($err, $all) = getAllowedResources("utg", "mhap",     $err, $all)   if (getGlobal("utgOverlapper") eq "mhap");
-    ($err, $all) = getAllowedResources("utg", "mmap",     $err, $all)   if (getGlobal("utgOverlapper") eq "minimap");
-    ($err, $all) = getAllowedResources("utg", "ovl",      $err, $all)   if (getGlobal("utgOverlapper") eq "ovl");
+    ($err, $all) = getAllowedResources("obt", "mhap",      $err, $all, 0)   if (getGlobal("obtOverlapper") eq "mhap");
+    ($err, $all) = getAllowedResources("obt", "mmap",      $err, $all, 0)   if (getGlobal("obtOverlapper") eq "minimap");
+    ($err, $all) = getAllowedResources("obt", "ovl",       $err, $all, 0)   if (getGlobal("obtOverlapper") eq "ovl");
 
-    ($err, $all) = getAllowedResources("",    "cor",      $err, $all);
+    ($err, $all) = getAllowedResources("utg", "mhap",      $err, $all, 0)   if (getGlobal("utgOverlapper") eq "mhap");
+    ($err, $all) = getAllowedResources("utg", "mmap",      $err, $all, 0)   if (getGlobal("utgOverlapper") eq "minimap");
+    ($err, $all) = getAllowedResources("utg", "ovl",       $err, $all, 0)   if (getGlobal("utgOverlapper") eq "ovl");
 
-    ($err, $all) = getAllowedResources("",    "ovb",      $err, $all);
-    ($err, $all) = getAllowedResources("",    "ovs",      $err, $all);
+    #  Usually set based on read length in CorrectReads.pm.  If defined by the user, run through configuration.
+    ($err, $all) = getAllowedResources("",    "cor",       $err, $all, 0)   if (getGlobal("corMemory") ne undef);
 
-    ($err, $all) = getAllowedResources("",    "red",      $err, $all);
-    ($err, $all) = getAllowedResources("",    "oea",      $err, $all);
+    ($err, $all) = getAllowedResources("",    "ovb",       $err, $all, 0);
+    ($err, $all) = getAllowedResources("",    "ovs",       $err, $all, 0);
 
-    ($err, $all) = getAllowedResources("",    "bat",      $err, $all);
+    ($err, $all) = getAllowedResources("",    "red",       $err, $all, 0);
+    ($err, $all) = getAllowedResources("",    "oea",       $err, $all, 0);
 
-    ($err, $all) = getAllowedResources("",    "cns",      $err, $all);
+    ($err, $all) = getAllowedResources("",    "bat",       $err, $all, 1)   if (uc(getGlobal("unitigger")) eq "BOGART");
+    ($err, $all) = getAllowedResources("",    "dbg",       $err, $all, 1)   if (uc(getGlobal("unitigger")) eq "WTDBG");
 
-    ($err, $all) = getAllowedResources("",    "gfa",      $err, $all);
+    ($err, $all) = getAllowedResources("",    "cns",       $err, $all, 0)   if (getGlobal("cnsMemory") ne undef);
+
+    ($err, $all) = getAllowedResources("",    "gfa",       $err, $all, 1);
 
     #  Check some minimums.
 
@@ -781,5 +890,3 @@ sub configureAssembler () {
     print STDERR "--\n";
     print STDERR $all;
 }
-
-
