@@ -56,8 +56,7 @@ require Exporter;
 
 @ISA    = qw(Exporter);
 @EXPORT = qw(stopAfter
-             skipStage
-             emitStage
+             resetIteration
              touch
              makeExecutable
              getInstallDirectory
@@ -77,17 +76,70 @@ require Exporter;
              caFailure);
 
 use strict;
+use warnings "all";
+no  warnings "uninitialized";
+
 use Config;            #  for @signame
 use Cwd qw(getcwd);
 use Carp qw(longmess);
 
 use POSIX ":sys_wait_h";  #  For waitpid(..., &WNOHANG)
+use File::Basename;
 use List::Util qw(min max);
 use File::Path 2.08 qw(make_path remove_tree);
 use File::Spec;
 
 use canu::Defaults;
-use canu::Report   qw(generateReport);
+
+
+
+
+
+#  Log that we've finished a task.
+
+sub logFinished ($$) {
+    my $dir       = shift @_;
+    my $startsecs = shift @_;
+
+    my $diskfree  = diskSpace(".");
+
+    my $warning = "  !!! WARNING !!!" if ($diskfree < 10);
+    my $elapsed = time() - $startsecs;
+    my $message;
+
+    my @fast;
+
+    push @fast, "lickety-split";
+    push @fast, "fast as lightning";
+    push @fast, "furiously fast";
+    push @fast, "like a bat out of hell";
+    push @fast, "in the blink of an eye";
+
+    my @slow;
+
+    push @slow, "fashionably late";
+    push @slow, "better late than never";
+    push @slow, "like watching paint dry";
+    push @slow, "at least I didn't crash";
+    push @slow, "it'll be worth it in the end";
+    push @slow, "no bitcoins found either";
+
+    my $rf = int(rand(scalar(@fast)));
+    my $rs = int(rand(scalar(@slow)));
+    my $rp = int(rand(100));
+
+    $message  = "$elapsed seconds" if ($elapsed  > 1);
+    $message  = "one second"       if ($elapsed == 1);
+    $message  = $fast[$rf]         if ($elapsed  < 1);
+
+    $message .= ", " . $slow[$rs]  if ((($elapsed > 1000)  && ($rp < 1)) ||
+                                       (($elapsed > 10000) && ($rp < 50)) ||
+                                       (($elapsed > 86400)));
+
+    print STDERR "\n";
+    print STDERR "-- Finished on ", scalar(localtime()), " ($message) with $diskfree GB free disk space$warning\n";
+    print STDERR "----------------------------------------\n";
+}
 
 
 
@@ -214,20 +266,9 @@ sub schedulerFinish ($$) {
         waitpid(shift @processesRunning, 0);
     }
 
+    logFinished($dir, $startsecs);
+
     chdir($cwd);
-
-    $diskfree = (defined($dir)) ? (diskSpace($dir)) : (0);
-
-    my $warning = "  !!! WARNING !!!" if ($diskfree < 10);
-    my $elapsed = time() - $startsecs;
-
-    $elapsed = "lickety-split"    if ($elapsed eq "0");
-    $elapsed = "$elapsed second"  if ($elapsed eq "1");
-    $elapsed = "$elapsed seconds" if ($elapsed  >  1);
-
-    print STDERR "\n";
-    print STDERR "-- Finished on ", scalar(localtime()), " ($elapsed) with $diskfree GB free disk space$warning\n";
-    print STDERR "----------------------------------------\n";
 }
 
 
@@ -269,22 +310,12 @@ sub stopAfter ($) {
 }
 
 
-sub emitStage ($$@) {
-    my $asm     = shift @_;
-    my $stage   = shift @_;
-    my $attempt = shift @_;
+sub resetIteration ($) {
+    my $stage = shift @_;
 
-    generateReport($asm);
+    print STDERR "-- Finished stage '$stage', reset canuIteration.\n"       if (defined($stage));
 
-    if (!defined($attempt)) {
-        print STDERR "-- Finished stage '$stage', reset canuIteration.\n";
-        setGlobal("canuIteration", 0);
-    }
-}
-
-
-sub skipStage ($$@) {
-    return(0);
+    setGlobal("canuIteration", 0);
 }
 
 
@@ -326,9 +357,9 @@ sub getJobIDShellCode () {
     $string .= "#  Discover the job ID to run, from either a grid environment variable and a\n";
     $string .= "#  command line offset, or directly from the command line.\n";
     $string .= "#\n";
-    $string .= "if [ x\$PBS_JOBID != x -a x\$$taskenv = x ]; then\n"   if (uc(getGlobal("gridEngine")) eq "PBSPRO");
-    $string .= "  $taskenv=1\n"                                        if (uc(getGlobal("gridEngine")) eq "PBSPRO");
-    $string .= "fi\n"                                                  if (uc(getGlobal("gridEngine")) eq "PBSPRO");
+    $string .= "if [ x\$PBS_JOBID != x -a x\$$taskenv = x ]; then\n"   if (uc(getGlobal("gridEngine")) eq "PBSPRO" || uc(getGlobal("gridEngine")) eq "PBS");
+    $string .= "  $taskenv=1\n"                                        if (uc(getGlobal("gridEngine")) eq "PBSPRO" || uc(getGlobal("gridEngine")) eq "PBS");
+    $string .= "fi\n"                                                  if (uc(getGlobal("gridEngine")) eq "PBSPRO" || uc(getGlobal("gridEngine")) eq "PBS");
     $string .= "if [ x\$$taskenv = x -o x\$$taskenv = xundefined -o x\$$taskenv = x0 ]; then\n";
     $string .= "  baseid=\$1\n";           #  Off grid
     $string .= "  offset=0\n";
@@ -388,32 +419,16 @@ sub getLimitShellCode () {
 }
 
 
-#  Used inside canu to find where binaries are located.  It uses uname to find OS, architecture and
-#  system name, then uses that to construct a path to binaries.  If a "pathMap" is defined, this is
-#  used to hardcode a path to a system name.
+#  Used inside canu to find where binaries are located.
 #
 sub getBinDirectory () {
     my $installDir = getInstallDirectory();
 
-    my $syst = `uname -s`;    chomp $syst;  #  OS implementation
-    my $arch = `uname -m`;    chomp $arch;  #  Hardware platform
-    my $name = `uname -n`;    chomp $name;  #  Name of the system
-
-    $arch = "amd64"  if ($arch eq "x86_64");
-    $arch = "ppc"    if ($arch eq "Power Macintosh");
+    my $syst = `uname -s`;                          chomp $syst;  #  OS implementation
+    my $arch = `uname -m | sed s/x86_64/amd64/`;    chomp $arch;  #  Hardware platform
 
     my $path = "$installDir/$syst-$arch/bin";
 
-    my $pathMap = getGlobal("pathMap");
-    if (defined($pathMap)) {
-        open(F, "< $pathMap") or caFailure("failed to open pathMap '$pathMap'", undef);
-        while (<F>) {
-            my ($n, $b) = split '\s+', $_;
-            $path = $b if ($name eq $n);
-        }
-        close(F);
-    }
-
     if (! -d "$path") {
         $path = $installDir;
     }
@@ -429,37 +444,56 @@ sub getBinDirectoryShellCode () {
     my $installDir = getInstallDirectory();
     my $string;
 
-    $string .= "syst=`uname -s`\n";
-    $string .= "arch=`uname -m`\n";
-    $string .= "name=`uname -n`\n";
-    $string .= "\n";
-    $string .= "if [ \"\$arch\" = \"x86_64\" ] ; then\n";
-    $string .= "  arch=\"amd64\"\n";
-    $string .= "fi\n";
-    $string .= "if [ \"\$arch\" = \"Power Macintosh\" ] ; then\n";
-    $string .= "  arch=\"ppc\"\n";
-    $string .= "fi\n";
-    $string .= "\n";
-    $string .= "bin=\"$installDir/\$syst-\$arch/bin\"\n";
-    $string .= "\n";
+    #  First, run any preExec command that might exist.
 
-    my $pathMap = getGlobal("pathMap");
-    if (defined($pathMap)) {
-        open(PM, "< $pathMap") or caFailure("failed to open pathMap '$pathMap'", undef);
-        while (<PM>) {
-            my ($n, $b) = split '\s+', $_;
-            $string .= "if [ \"\$name\" = \"$n\" ] ; then\n";
-            $string .= "  bin=\"$b\"\n";
-            $string .= "fi\n";
-        }
-        close(PM);
+    if (defined(getGlobal("preExec"))) {
+        $string .= "#  Pre-execution commands.\n";
+        $string .= "\n";
+        $string .= getGlobal('preExec') . "\n";
         $string .= "\n";
     }
 
+    #  Then, setup and report paths.
+
+    my $javaPath = getGlobal("java");
+    my $canu     = "\$bin/" . basename($0);
+
+    $string .= "\n";
+    $string .= "#  Path to Canu.\n";
+    $string .= "\n";
+    $string .= "syst=`uname -s`\n";
+    $string .= "arch=`uname -m | sed s/x86_64/amd64/`\n";
+    $string .= "\n";
+    $string .= "bin=\"$installDir/\$syst-\$arch/bin\"\n";
+    $string .= "\n";
     $string .= "if [ ! -d \"\$bin\" ] ; then\n";
     $string .= "  bin=\"$installDir\"\n";
     $string .= "fi\n";
     $string .= "\n";
+    $string .= "#  Report paths.\n";
+    $string .= "\n";
+    $string .= "echo \"\"\n";
+    $string .= "echo \"Found perl:\"\n";
+    $string .= "echo \"  \" `which perl`\n";
+    $string .= "echo \"  \" `perl --version | grep version`\n";
+    $string .= "echo \"\"\n";
+    $string .= "echo \"Found java:\"\n";
+    $string .= "echo \"  \" `which $javaPath`\n";
+    $string .= "echo \"  \" `$javaPath -showversion 2>&1 | head -n 1`\n";
+    $string .= "echo \"\"\n";
+    $string .= "echo \"Found canu:\"\n";
+    $string .= "echo \"  \" $canu\n";
+    $string .= "echo \"  \" `$canu -version`\n";
+    $string .= "echo \"\"\n";
+    $string .= "\n";
+    $string .= "\n";
+    $string .= "#  Environment for any object storage.\n";
+    $string .= "\n";
+    $string .= "export CANU_OBJECT_STORE_CLIENT="    . getGlobal("objectStoreClient")    . "\n";
+    $string .= "export CANU_OBJECT_STORE_NAMESPACE=" . getGlobal("objectStoreNameSpace") . "\n";
+    $string .= "export CANU_OBJECT_STORE_PROJECT="   . getGlobal("objectStoreProject")   . "\n";
+    $string .= "\n";
+    $string .= "\n";
 
     return($string);
 }
@@ -478,14 +512,23 @@ sub getBinDirectoryShellCode () {
 #  Note that canu does minimal cleanup.
 #
 
-sub setWorkDirectory () {
+sub setWorkDirectory ($$) {
+    my $asm     = shift @_;
+    my $rootdir = shift @_;
+
+    #  Set the initial directory based on various rules.
+    #
+    #  For the canu executive, in grid mode, both setWorkDirectoryShellCode and
+    #  this (in that order) are called.  TEST is assuming that all (non-executive)
+    #  compute jobs are run as arrays.
 
     if    ((getGlobal("objectStore") eq "TEST") && (defined($ENV{"JOB_ID"}))) {
         my $jid = $ENV{'JOB_ID'};
-        my $tid = $ENV{'SGE_TASK_ID'};
+        my $tid = $ENV{'SGE_TASK_ID'};   #  'undefined' since this isn't an array job.
 
-        make_path("/assembly/COMPUTE/job-$jid-$tid");
-        chdir    ("/assembly/COMPUTE/job-$jid-$tid");
+        remove_tree("/assembly/objectstore/job-$jid");
+        make_path  ("/assembly/objectstore/job-$jid");
+        chdir      ("/assembly/objectstore/job-$jid");
     }
 
     elsif (getGlobal("objectStore") eq "DNANEXUS") {
@@ -494,6 +537,18 @@ sub setWorkDirectory () {
     elsif (getGlobal("gridEngine") eq "PBSPRO") {
         chdir($ENV{"PBS_O_WORKDIR"})   if (exists($ENV{"PBS_O_WORKDIR"}));
     }
+
+    #  Now move into the assembly directory.
+
+    if (defined($rootdir)) {
+        make_path($rootdir)  if (! -d $rootdir);
+        chdir($rootdir);
+    }
+
+    #  And save some pieces we need when we quit.
+
+    setGlobal("onExitDir", getcwd());
+    setGlobal("onExitNam", $asm);
 }
 
 
@@ -506,15 +561,19 @@ sub setWorkDirectoryShellCode ($) {
         $code .= "if [ z\$SGE_TASK_ID != z ] ; then\n";
         $code .= "  jid=\$JOB_ID\n";
         $code .= "  tid=\$SGE_TASK_ID\n";
-        $code .= "  mkdir -p /assembly/COMPUTE/job-\$jid-\$tid/$path\n";
-        $code .= "  cd       /assembly/COMPUTE/job-\$jid-\$tid/$path\n";
-        $code .= "  echo IN  /assembly/COMPUTE/job-\$jid-\$tid/$path\n";
+        $code .= "  if [ x\$tid != xundefined ] ; then\n";
+        $code .= "    rm   -rf /assembly/objectstore/job-\$jid-\$tid/\n";
+        $code .= "    mkdir -p /assembly/objectstore/job-\$jid-\$tid/$path\n";
+        $code .= "    cd       /assembly/objectstore/job-\$jid-\$tid/$path\n";
+        $code .= "  fi\n";
         $code .= "fi\n";
     }
+
     elsif (getGlobal("objectStore") eq "DNANEXUS") {
         #  You're probably fine running in some random location, but if there is faster disk
         #  available, move there.
     }
+
     elsif (getGlobal("gridEngine") eq "PBSPRO") {
         $code .= "if [ z\$PBS_O_WORKDIR != z ] ; then\n";
         $code .= "  cd \$PBS_O_WORKDIR\n";
@@ -623,9 +682,7 @@ sub submitScript ($$) {
 
     return   if (($jobHold eq undef) && (exists($ENV{getGlobal("gridEngineJobID")})));
 
-    #  Find the next available output file.
-
-    make_path("canu-scripts")  if (! -d "canu-scripts");  #  Done in canu.pl, just being paranoid
+    #  Figure out the name of the script we want to be making, and a place for it to write output.
 
     my $idx = "01";
 
@@ -633,36 +690,33 @@ sub submitScript ($$) {
         $idx++;
     }
 
-    my $outName   = "canu-scripts/canu.$idx.out";
     my $script    = "canu-scripts/canu.$idx.sh";
+    my $scriptOut = "canu-scripts/canu.$idx.out";
 
     #  Make a script for us to submit.
 
     open(F, "> $script") or caFailure("failed to open '$script' for writing", undef);
     print F "#!" . getGlobal("shell") . "\n";
-    print F "\n"                                                                        if (getGlobal("gridEngine") eq "SGE");
+    print F "\n";
     print F "#  Attempt to (re)configure SGE.  For unknown reasons, jobs submitted\n"   if (getGlobal("gridEngine") eq "SGE");
     print F "#  to SGE, and running under SGE, fail to read the shell init scripts,\n"  if (getGlobal("gridEngine") eq "SGE");
     print F "#  and so they don't set up SGE (or ANY other paths, etc) properly.\n"     if (getGlobal("gridEngine") eq "SGE");
     print F "#  For the record, interactive logins (qlogin) DO set the environment.\n"  if (getGlobal("gridEngine") eq "SGE");
-    print F "\n";
-    print F "if [ \"x\$SGE_ROOT\" != \"x\" ]; then \n"                                  if (getGlobal("gridEngine") eq "SGE");
+    print F "\n"                                                                        if (getGlobal("gridEngine") eq "SGE");
+    print F "if [ \"x\$SGE_ROOT\" != \"x\" -a \\\n"                                     if (getGlobal("gridEngine") eq "SGE");
+    print F "     -e  \$SGE_ROOT/\$SGE_CELL/common/settings.sh ]; then\n"               if (getGlobal("gridEngine") eq "SGE");
     print F "  . \$SGE_ROOT/\$SGE_CELL/common/settings.sh\n"                            if (getGlobal("gridEngine") eq "SGE");
     print F "fi\n"                                                                      if (getGlobal("gridEngine") eq "SGE");
     print F "\n";
-    print F "#  On the off chance that there is a pathMap, and the host we\n";
-    print F "#  eventually get scheduled on doesn't see other hosts, we decide\n";
-    print F "#  at run time where the binary is.\n";
-    print F "\n";
     print F getBinDirectoryShellCode();
     print F "\n";
     print F setWorkDirectoryShellCode(".");
     print F "\n";
     print F "rm -f canu.out\n";
-    print F "ln -s canu-scripts/canu.$idx.out canu.out\n";
+    print F "ln -s $scriptOut canu.out\n";
     print F "\n";
     print F "/usr/bin/env perl \\\n";
-    print F "\$bin/canu " . getCommandLineOptions() . " canuIteration=" . getGlobal("canuIteration") . "\n";
+    print F "\$bin/" . basename($0) . " " . getCommandLineOptions() . " canuIteration=" . getGlobal("canuIteration") . "\n";
     close(F);
 
     makeExecutable("$script");
@@ -675,34 +729,53 @@ sub submitScript ($$) {
 
     #  The canu.pl script isn't expected to take resources.  We'll default to 4gb and one thread.
 
-    my $mem = 4;
-    my $thr = 1;
+    my $mem = getGlobal("executiveMemory");
+    my $thr = getGlobal("executiveThreads");
 
-    #  However, the sequential overlap store is still built from within the canu process.
-
-    if (getGlobal("ovsMethod") eq "sequential") {
-        $mem = getGlobal("ovsMemory");
-        $mem = $2  if ($mem =~ m/^(\d+)-(\d+)$/);
-    }
-
-    $memOption = buildMemoryOption($mem, 1);
+    $memOption = buildMemoryOption($mem, $thr);
     $thrOption = buildThreadOption($thr);
 
     $gridOpts  = $jobHold;
     $gridOpts .= " "                                  if (defined($gridOpts));
-    $gridOpts .= $memOption                           if (defined($memOption));
+    # LSF takes the first argument to specify them in reverse order of preference, otherwise append
+    if (uc(getGlobal("gridEngine")) eq "LSF") {
+       $gridOpts .= getGlobal("gridOptionsExecutive")    if (defined(getGlobal("gridOptionsExecutive")));
+       $gridOpts .= " "                                  if (defined($gridOpts));
+       $gridOpts .= getGlobal("gridOptions")             if (defined(getGlobal("gridOptions")));
+       $gridOpts .= " "                                  if (defined($gridOpts));
+    }
+    # For DNANEXUS, we won't specify memory when resubmitting canu.  It will use the same setup
+    # as was specified when it was first submitted.
+    if (uc(getGlobal("gridEngine")) ne "DNANEXUS") {
+        $gridOpts .= $memOption                           if (defined($memOption));
+    }
     $gridOpts .= " "                                  if (defined($gridOpts));
     $gridOpts .= $thrOption                           if (defined($thrOption));
-    $gridOpts .= " "                                  if (defined($gridOpts));
-    $gridOpts .= getGlobal("gridOptions")             if (defined(getGlobal("gridOptions")));
-    $gridOpts .= " "                                  if (defined($gridOpts));
-    $gridOpts .= getGlobal("gridOptionsExecutive")    if (defined(getGlobal("gridOptionsExecutive")));
+    if (uc(getGlobal("gridEngine")) ne "LSF") {
+       $gridOpts .= " "                                  if (defined($gridOpts));
+       $gridOpts .= getGlobal("gridOptions")             if (defined(getGlobal("gridOptions")));
+       $gridOpts .= " "                                  if (defined($gridOpts));
+       $gridOpts .= getGlobal("gridOptionsExecutive")    if (defined(getGlobal("gridOptionsExecutive")));
+    }
 
     my $submitCommand        = getGlobal("gridEngineSubmitCommand");
     my $nameOption           = getGlobal("gridEngineNameOption");
     my $outputOption         = getGlobal("gridEngineOutputOption");
 
-    my $qcmd = "$submitCommand $gridOpts $nameOption '$jobName' $outputOption $outName $script";
+    my $qcmd = "$submitCommand $gridOpts $nameOption '$jobName'";
+    $qcmd .= " $outputOption $scriptOut" if defined($outputOption);
+
+    # on dna nexus we don't submit a script and there is no logging
+    # any parameters get passed through the -i option
+    # the fetch_and_run call is to a function in the app which will downloaded the requested shell script and execute it
+    if (uc(getGlobal("gridEngine")) eq "DNANEXUS") {
+        $qcmd .= " -iscript_name:string=\"canu.sh\" -icanu_path:string=\"\" \\\n";
+        $qcmd .= " -icanu_iteration:int=" . getGlobal("canuIteration") . " \\\n";
+        $qcmd .= " -icanu_iteration_max:int=" . getGlobal("canuIterationMax") . " \\\n";
+        $qcmd .= " fetch_and_run \\\n";
+    } else {
+        $qcmd .= "  $script";
+    }
 
     runCommand(getcwd(), $qcmd) and caFailure("Failed to submit script", undef);
 
@@ -729,10 +802,18 @@ sub buildGridArray ($$$$) {
     #  PBSPro requires array jobs to have bgn < end.  When $bgn == $end, we
     #  just remove the array qualifier.  But only if this option is setting
     #  the number of jobs, not if it is setting the name.
+    #  New versions of PBS have this behavior too
 
-    if (uc(getGlobal("gridEngine")) eq "PBSPRO") {
-        $opt = ""  if (($bgn == $end) && ($opt =~ m/ARRAY_JOBS/));
-        $off = "";
+    if (uc(getGlobal("gridEngine")) eq "PBSPRO" || uc(getGlobal("gridEngine")) eq "PBS") {
+        $opt = ""   if (($bgn == $end) && ($opt =~ m/ARRAY_JOBS/));
+        $off = $bgn if (($bgn == $end) && ($opt =~ m/ARRAY_JOBS/));
+    }
+    # DNA nexus doesn't have arrays and only supports 1 job, which we use to pass the identifier
+    # Set the offset to blank since it is not supported as well
+    if (uc(getGlobal("gridEngine")) eq "DNANEXUS" && ($bgn == $end) && ($opt =~ m/ARRAY_JOBS/)) {
+           my $jid = $bgn + $off;
+           $opt =~ s/ARRAY_JOBS/$jid/g;
+           $off = "";
     }
 
     #  Further, PBS/Torque won't let scripts be passed options unless they
@@ -740,7 +821,6 @@ sub buildGridArray ($$$$) {
 
     if (uc(getGlobal("gridEngine")) eq "PBS") {
         $off = "-F \"$off\"";
-        $off = "";
     }
 
     $opt =~ s/ARRAY_NAME/$name/g;        #  Replace ARRAY_NAME with 'job name'
@@ -827,6 +907,11 @@ sub buildMemoryOption ($$) {
         $m = $m * 1024 * 1024   if (getGlobal("gridEngineMemoryUnits") eq "k");
         $u = "";
     }
+    if (uc(getGlobal("gridEngine")) eq "DNANEXUS") {
+       $m = canu::Grid_DNANexus::getDNANexusInstance($m, $t);
+       $u = "";
+    }
+
 
     $r =  getGlobal("gridEngineMemoryOption");
     $r =~ s/MEMORY/${m}${u}/g;
@@ -926,7 +1011,9 @@ sub buildGridJob ($$$$$$$$$) {
     print F "  $opts \\\n"  if (defined($opts));
     print F "  $nameOption \"$jobName\" \\\n";
     print F "  $arrayOpt \\\n";
-    print F "  ./$script.sh $arrayOff \\\n";
+    print F " -- " if (uc(getGlobal("gridEngine")) eq "PBSPRO");
+    print F " -iscript_name:string=\"$script.sh\" -icanu_path:string=\"$path\" fetch_and_run \\\n" if (uc(getGlobal("gridEngine")) eq "DNANEXUS");
+    print F "  ./$script.sh $arrayOff \\\n"                                                        if (uc(getGlobal("gridEngine")) ne "DNANEXUS");
     print F "> ./$script.jobSubmit-$idx.out 2>&1\n";
     close(F);
 
@@ -997,7 +1084,7 @@ sub convertToJobRange (@) {
 
     my $l = getGlobal("gridEngineArrayMaxJobs") - 1;
 
-    if ($l > 0) {
+    if ($l >= 0) {
         @jobsA = @jobs;
         undef @jobs;
 
@@ -1024,6 +1111,23 @@ sub convertToJobRange (@) {
 
 
 
+sub countJobsInRange (@) {
+    my @jobs  = @_;
+    my $nJobs = 0;
+
+    foreach my $j (@jobs) {
+        if ($j =~ m/^(\d+)-(\d+)$/) {
+            $nJobs += $2 - $1 + 1;
+        } else {
+            $nJobs++;
+        }
+    }
+
+    return($nJobs);
+}
+
+
+
 #  Expects
 #    job type ("ovl", etc)
 #    output directory
@@ -1046,11 +1150,21 @@ sub submitOrRunParallelJob ($$$$@) {
     my $dsk          = getGlobal("${jobType}StageSpace");
 
     my @jobs         = convertToJobRange(@_);
+    my $nJobs        = countJobsInRange(@jobs);
+
+    my $runDirectly  = 0;
 
     #  The script MUST be executable.
 
     makeExecutable("$path/$script.sh");
 
+    #  If the job can fit in the task running the executive, run it right here.
+
+    if (($nJobs * $mem + 0.5 <= getGlobal("executiveMemory")) &&
+        ($nJobs * $thr       <= getGlobal("executiveThreads"))) {
+        $runDirectly = 1;
+    }
+
     #  Report what we're doing.
 
     #my $t = localtime();
@@ -1065,8 +1179,8 @@ sub submitOrRunParallelJob ($$$$@) {
     #  Assuming grid jobs die on each attempt:
     #    0) canu run from the command line submits iteration 1; canuIteration is NOT incremented
     #       because no parallel jobs have been submitted.
-    #    1) Iteration 1 - canu.pl submits jobs, increments the interation count, and submits itself as iteration 2
-    #    2) Iteration 2 - canu.pl submits jobs, increments the interation count, and submits itself as iteration 3
+    #    1) Iteration 1 - canu.pl submits jobs, increments the iteration count, and submits itself as iteration 2
+    #    2) Iteration 2 - canu.pl submits jobs, increments the iteration count, and submits itself as iteration 3
     #    3) Iteration 3 - canu.pl fails with the error below
     #
     #  If the jobs succeed in Iteration 2, the canu in iteration 3 will pass the Check(), never call
@@ -1096,7 +1210,6 @@ sub submitOrRunParallelJob ($$$$@) {
 
     setGlobal("canuIteration", getGlobal("canuIteration") + 1);
 
-
     #  If 'gridEngineJobID' environment variable exists (SGE: JOB_ID; LSF: LSB_JOBID) then we are
     #  currently running under grid crontrol.  If so, run the grid command to submit more jobs, then
     #  submit ourself back to the grid.  If not, tell the user to run the grid command by hand.
@@ -1106,7 +1219,8 @@ sub submitOrRunParallelJob ($$$$@) {
     if (defined(getGlobal("gridEngine")) &&
         (getGlobal("useGrid") eq "1") &&
         (getGlobal("useGrid$jobType") eq "1") &&
-        (exists($ENV{getGlobal("gridEngineJobID")}))) {
+        (exists($ENV{getGlobal("gridEngineJobID")})) &&
+        ($runDirectly == 0)) {
         my @jobsSubmitted;
 
         print STDERR "--\n";
@@ -1163,6 +1277,7 @@ sub submitOrRunParallelJob ($$$$@) {
                 }
 
                 if (uc(getGlobal("gridEngine")) eq "DNANEXUS") {
+                   $jobName = $_;
                 }
             }
             close(F);
@@ -1191,7 +1306,10 @@ sub submitOrRunParallelJob ($$$$@) {
         }
 
         if (uc(getGlobal("gridEngine")) eq "PBS") {
-            $jobHold = "-W depend=afteranyarray:" . join ":", @jobsSubmitted;
+            # new PBS versions dont have 1-task arrays like PBSPro but still have afteranyarray (which doesn't work on a not-array task)
+            # so we need to check if we are waiting for a regular job or array
+            my $holdType = (join ":", @jobsSubmitted)  =~ m/^(\d+)\[(.*)\]/ ? "afteranyarray" : "afterany";
+            $jobHold = "-W depend=$holdType:" . join ":", @jobsSubmitted;
         }
 
         if (uc(getGlobal("gridEngine")) eq "PBSPRO") {
@@ -1203,7 +1321,7 @@ sub submitOrRunParallelJob ($$$$@) {
         }
 
         if (uc(getGlobal("gridEngine")) eq "DNANEXUS") {
-            $jobHold = "...whatever magic needed to hold the job until all jobs in @jobsSubmitted are done...";
+            $jobHold = "--depends-on " . join " ", @jobsSubmitted;
         }
 
         submitScript($asm, $jobHold);
@@ -1218,22 +1336,28 @@ sub submitOrRunParallelJob ($$$$@) {
     if (defined(getGlobal("gridEngine")) &&
         (getGlobal("useGrid") ne "0") &&
         (getGlobal("useGrid$jobType") eq "1") &&
-        (! exists($ENV{getGlobal("gridEngineJobID")}))) {
+        (! exists($ENV{getGlobal("gridEngineJobID")})) &&
+        ($runDirectly == 0)) {
+        my $cwd = getcwd();
+        my $s   = (scalar(@jobs) == 1) ? "" : "s";
+
         print STDERR "\n";
-        print STDERR "Please run the following commands to submit jobs to the grid for execution using $mem gigabytes memory and $thr threads:\n";
+        print STDERR "Please run the following command$s to submit tasks to the grid for execution.\n";
+        print STDERR "Each task will use $mem gigabytes memory and $thr threads.\n";
         print STDERR "\n";
+        print STDERR "  cd $cwd/$path\n";
 
         purgeGridJobSubmitScripts($path, $script);
 
         foreach my $j (@jobs) {
-            my  $cwd = getcwd();
             my ($cmd, $jobName) = buildGridJob($asm, $jobType, $path, $script, $mem, $thr, $dsk, $j, undef);
 
-            print "  $cwd/$path/$cmd.sh\n";
+            print "  ./$cmd.sh\n";
         }
 
         print STDERR "\n";
-        print STDERR "When all jobs complete, restart canu as before.\n";
+        print STDERR "When all tasks are finished, restart canu as before.  The output of the grid\n";
+        print STDERR "submit command$s will be in *jobSubmit*out.\n";
         print STDERR "\n";
 
         exit(0);
@@ -1316,6 +1440,7 @@ sub reportRunError ($) {
 
     #  The rest is rather straightforward at least.
 
+    print STDERR "\n";
     print STDERR "ERROR:\n";
 
     if      ($rc ==  -1) {
@@ -1369,18 +1494,7 @@ sub runCommand ($$) {
 
     my $rc = 0xffff & system($cmd);
 
-    $diskfree = diskSpace(".");
-
-    my $warning = "  !!! WARNING !!!" if ($diskfree < 10);
-    my $elapsed = time() - $startsecs;
-
-    $elapsed = "lickety-split"    if ($elapsed eq "0");
-    $elapsed = "$elapsed second"  if ($elapsed eq "1");
-    $elapsed = "$elapsed seconds" if ($elapsed  >  1);
-
-    print STDERR "\n";
-    print STDERR "-- Finished on ", scalar(localtime()), " ($elapsed) with $diskfree GB free disk space$warning\n";
-    print STDERR "----------------------------------------\n";
+    logFinished(".", $startsecs);
 
     chdir($cwd);
 
@@ -1459,6 +1573,9 @@ sub caExit ($$) {
     my  $log     = shift @_;
     my  $version = getGlobal("version");
 
+    $msg = undef   if ($msg eq "");
+    $log = undef   if ($log eq "");
+
     print STDERR "\n";
     print STDERR "ABORT:\n";
     print STDERR "ABORT: $version\n";
@@ -1479,9 +1596,9 @@ sub caExit ($$) {
         print STDERR "ABORT: Last 50 lines of the relevant log file ($log):\n";
         print STDERR "ABORT:\n";
 
-        open(Z, "tail -n 50 $log");
+        open(Z, "tail -n 50 $log |");
         while (<Z>) {
-            print STDERR "ABORT: $_";
+            print STDERR "ABORT:   $_";
         }
         close(Z);
 
@@ -1521,7 +1638,7 @@ sub caFailure ($$) {
         print STDERR "CRASH: Last 50 lines of the relevant log file ($log):\n";
         print STDERR "CRASH:\n";
 
-        open(Z, "tail -n 50 $log");
+        open(Z, "tail -n 50 $log |");
         while (<Z>) {
             print STDERR "CRASH: $_";
         }
