@@ -35,11 +35,16 @@ require Exporter;
 @EXPORT = qw(detectSGE configureSGE);
 
 use strict;
+use warnings "all";
+no  warnings "uninitialized";
 
 use canu::Defaults;
-use canu::Grid;
 use canu::Execution;
 
+use canu::Grid "formatAllowedResources";
+
+
+
 sub detectSGE () {
 
     return   if ( defined(getGlobal("gridEngine")));
