@@ -40,17 +40,21 @@ package canu::OverlapBasedTrimming;
 require Exporter;
 
 @ISA    = qw(Exporter);
-@EXPORT = qw(qualTrimReads dedupeReads trimReads splitReads dumpReads);
+@EXPORT = qw(qualTrimReads dedupeReads trimReads splitReads loadTrimmedReads dumpTrimmedReads);
 
 use strict;
+use warnings "all";
+no  warnings "uninitialized";
 
 use File::Path 2.08 qw(make_path remove_tree);
 
 use canu::Defaults;
 use canu::Execution;
-use canu::Gatekeeper;
+
+use canu::SequenceStore;
 use canu::Report;
-use canu::HTML;
+use canu::Output;
+
 use canu::Grid_Cloud;
 
 
@@ -60,18 +64,17 @@ sub trimReads ($) {
     my $cmd;
     my $path   = "trimming/3-overlapbasedtrimming";
 
-    goto allDone   if (skipStage($asm, "obt-trimReads") == 1);
     goto allDone   if (fileExists("trimming/3-overlapbasedtrimming/$asm.1.trimReads.clear"));
 
     make_path($path)  if (! -d $path);
 
-    fetchStore("./trimming/$asm.ovlStore");
+    fetchOvlStore($asm, "trimming");
 
     #  Previously, we'd pick the error rate used by unitigger.  Now, we don't know unitigger here,
     #  and require an obt specific error rate.
 
     $cmd  = "$bin/trimReads \\\n";
-    $cmd .= "  -G  ../$asm.gkpStore \\\n";
+    $cmd .= "  -S  ../../$asm.seqStore \\\n";
     $cmd .= "  -O  ../$asm.ovlStore \\\n";
     $cmd .= "  -Co ./$asm.1.trimReads.clear \\\n";
     $cmd .= "  -e  " . getGlobal("obtErrorRate") . " \\\n";
@@ -105,8 +108,8 @@ sub trimReads ($) {
 
 
     if (0) {
-        $cmd  = "$bin/gatekeeperDumpFASTQ \\\n";
-        $cmd .= "  -G ../$asm.gkpStore \\\n";
+        $cmd  = "$bin/sqStoreDumpFASTQ \\\n";
+        $cmd .= "  -S ../../$asm.seqStore \\\n";
         $cmd .= "  -c ./$asm.1.trimReads.clear \\\n";
         $cmd .= "  -o ./$asm.1.trimReads.trimmed \\\n";
         $cmd .= ">    ./$asm.1.trimReads.trimmed.err 2>&1";
@@ -117,8 +120,8 @@ sub trimReads ($) {
     }
 
   finishStage:
-    emitStage($asm, "obt-trimReads");
-    buildHTML($asm, "obt");
+    generateReport($asm);
+    resetIteration("obt-trimReads");
 
   allDone:
 }
@@ -131,12 +134,12 @@ sub splitReads ($) {
     my $cmd;
     my $path   = "trimming/3-overlapbasedtrimming";
 
-    goto allDone   if (skipStage($asm, "obt-splitReads") == 1);
     goto allDone   if (fileExists("trimming/3-overlapbasedtrimming/$asm.2.splitReads.clear"));
 
     make_path($path)  if (! -d $path);
 
-    fetchStore("./trimming/$asm.ovlStore");
+    fetchOvlStore($asm, "trimming");
+
     fetchFile("./trimming/3-overlapbasedtrimming/$asm.1.trimReads.clear");
 
     my $erate  = getGlobal("obtErrorRate");  #  Was this historically
@@ -144,7 +147,7 @@ sub splitReads ($) {
     #$cmd .= "  -mininniepair 0 -minoverhanging 0 \\\n" if (getGlobal("doChimeraDetection") eq "aggressive");
 
     $cmd  = "$bin/splitReads \\\n";
-    $cmd .= "  -G  ../$asm.gkpStore \\\n";
+    $cmd .= "  -S  ../../$asm.seqStore \\\n";
     $cmd .= "  -O  ../$asm.ovlStore \\\n";
     $cmd .= "  -Ci ./$asm.1.trimReads.clear \\\n"       if (-e "trimming/3-overlapbasedtrimming/$asm.1.trimReads.clear");
     #$cmd .= "  -Cm ./$asm.max.clear \\\n"               if (-e "trimming/3-overlapbasedtrimming/$asm.max.clear");
@@ -176,8 +179,8 @@ sub splitReads ($) {
     addToReport("splitting", $report);
 
     if (0) {
-        $cmd  = "$bin/gatekeeperDumpFASTQ \\\n";
-        $cmd .= "  -G ../$asm.gkpStore \\\n";
+        $cmd  = "$bin/sqStoreDumpFASTQ \\\n";
+        $cmd .= "  -S ../../$asm.seqStore \\\n";
         $cmd .= "  -c ./$asm.2.splitReads.clear \\\n";
         $cmd .= "  -o ./$asm.2.splitReads.trimmed \\\n";
         $cmd .= ">    ./$asm.2.splitReads.trimmed.err 2>&1";
@@ -188,57 +191,142 @@ sub splitReads ($) {
     }
 
   finishStage:
-    emitStage($asm, "obt-splitReads");
-    buildHTML($asm, "obt");
+    generateReport($asm);
+    resetIteration("obt-splitReads");
 
   allDone:
 }
 
 
 
-sub dumpReads ($) {
+sub loadTrimmedReads ($) {
     my $asm    = shift @_;
     my $bin    = getBinDirectory();
     my $cmd;
     my $path   = "trimming/3-overlapbasedtrimming";
     my $inp;
 
-    goto allDone   if (skipStage($asm, "obt-dumpReads") == 1);
-    goto allDone   if (sequenceFileExists("$asm.trimmedReads"));
+    goto allDone   if (getNumberOfBasesInStore($asm, "utg") > 0);
 
     make_path($path)  if (! -d $path);
 
     fetchFile("./trimming/3-overlapbasedtrimming/$asm.1.trimReads.clear");
     fetchFile("./trimming/3-overlapbasedtrimming/$asm.2.splitReads.clear");
 
-    $inp = "./3-overlapbasedtrimming/$asm.1.trimReads.clear"   if (-e "$path/$asm.1.trimReads.clear");
-    $inp = "./3-overlapbasedtrimming/$asm.2.splitReads.clear"  if (-e "$path/$asm.2.splitReads.clear");
+    $inp = "./$asm.1.trimReads.clear"   if (-e "$path/$asm.1.trimReads.clear");
+    $inp = "./$asm.2.splitReads.clear"  if (-e "$path/$asm.2.splitReads.clear");
 
-    caFailure("dumping trimmed reads failed; no 'clear' input", "trimming/$asm.trimmedReads.err")  if (!defined($inp));
+    caFailure("loading trimmed reads failed; no 'clear' input", "trimming/$asm.trimmedReads.err")  if (!defined($inp));
 
-    $cmd  = "$bin/gatekeeperDumpFASTQ -fasta -nolibname \\\n";
-    $cmd .= "  -G ./$asm.gkpStore \\\n";
+    $cmd  = "$bin/loadTrimmedReads \\\n";
+    $cmd .= "  -S ../../$asm.seqStore \\\n";
     $cmd .= "  -c $inp \\\n";
-    $cmd .= "  -o ../$asm.trimmedReads.gz \\\n";     #  Adds .fasta
-    $cmd .= ">    ../$asm.trimmedReads.err 2>&1";
+    $cmd .= "> ./$asm.loadtrimmedReads.err 2>&1";
 
-    if (runCommand("trimming", $cmd)) {
-        caFailure("dumping trimmed reads failed", "./$asm.trimmedReads.err");
+    if (runCommand($path, $cmd)) {
+        caFailure("loading clear ranges failed", "$path/$asm.loadTrimmedReads.err");
     }
 
-    unlink("./$asm.trimmedReads.err");
+    unlink("$path/$asm.trimmedReads.err");
 
-    stashFile("./$asm.trimmedReads.fasta.gz");
+    #  Save results.
 
-    remove_tree("trimming/$asm.ovlStore")   if (getGlobal("saveOverlaps") eq "0");
+    stashSeqStore($asm);
+
+    #  Report reads.
+
+    addToReport("utgSeqStore", generateReadLengthHistogram("utg", $asm));
+
+    #stashFile("./$asm.trimmedReads.fasta.gz");
+
+    if (getGlobal("saveOverlaps") eq "0") {
+        print STDERR "--\n";
+        print STDERR "-- Purging overlaps used for trimming.\n";
+
+        remove_tree("trimming/$asm.ovlStore")
+    } else {
+        print STDERR "--\n";
+        print STDERR "-- Overlaps used for trimming saved.\n";
+    }
 
   finishStage:
-    emitStage($asm, "obt-dumpReads");
-    buildHTML($asm, "obt");
+    generateReport($asm);
+    resetIteration("obt-dumpReads");
+
+  allDone:
+    stopAfter("readTrimming");
+}
+
+
+
+sub dumpTrimmedReads ($) {
+    my $asm     = shift @_;
+    my $bin     = getBinDirectory();
+    my $cmd;
+
+    goto allDone   if (fileExists("$asm.trimmedReads.fasta.gz"));
+    goto allDone   if (fileExists("$asm.trimmedReads.fastq.gz"));
+    goto allDone   if (getGlobal("saveReads") == 0);
+
+    #  We need to skip this entire function if trimmed reads were not computed.
+    #  Otherwise, we incorrectly declare that all reads were trimmed out
+    #  and halt the assembly.
+    #
+    #  In cloud mode, we dump reads right after loading them, and to load them,
+    #  the 3-overlapbasedtrimming directory must exist, so we use that to decide if trimming
+    #  was attempted.
+
+    return         if (! -d "trimming/3-overlapbasedtrimming");
+
+    #  If no trimmed reads exist, don't bother trying to dump them.
+
+    if (getNumberOfReadsInStore($asm, "utg") > 0) {
+        $cmd  = "$bin/sqStoreDumpFASTQ \\\n";
+        $cmd .= "  -trimmed \\\n";
+        $cmd .= "  -S ./$asm.seqStore \\\n";
+        $cmd .= "  -o ./$asm.trimmedReads.gz \\\n";
+        $cmd .= "  -fasta \\\n";
+        $cmd .= "  -nolibname \\\n";
+        $cmd .= "> ./$asm.trimmedReads.fasta.err 2>&1";
+
+        if (runCommand(".", $cmd)) {
+            caExit("failed to output trimmed reads", "./$asm.trimmedReads.fasta.err");
+        }
+
+        unlink "./$asm.trimmedReads.fasta.err";
+
+        stashFile("$asm.trimmedReads.fasta.gz");
+    }
+
+    #  If the trimmed reads file exists, report so.
+    #  Otherwise, report no trimmed reads, and generate fake outputs so we terminate.
+
+    my $out;
+
+    $out = "$asm.trimmedReads.fasta.gz"   if (fileExists("$asm.trimmedReads.fasta.gz"));
+    $out = "$asm.trimmedReads.fastq.gz"   if (fileExists("$asm.trimmedReads.fastq.gz"));
+
+    if (defined($out)) {
+        print STDERR "--\n";
+        print STDERR "-- Trimmed reads saved in '$out'.\n";
+    } else {
+        print STDERR "--\n";
+        print STDERR "-- Yikes!  No trimmed reads generated!\n";
+        print STDERR "-- Can't proceed!\n";
+        print STDERR "--\n";
+        print STDERR "-- Generating empty outputs.\n";
+
+        runCommandSilently(".", "gzip -1vc < /dev/null > $asm.trimmedReads.gz   2> /dev/null", 0)   if (! -e "$asm.trimmedReads.gz");
+
+        stashFile("$asm.trimmedReads.fasta.gz");
+
+        generateOutputs($asm);
+    }
+
+  finishStage:
+    generateReport($asm);
+    resetIteration("cor-dumpTrimmedReads");
 
   allDone:
-    print STDERR "--\n";
-    print STDERR "-- Trimmed reads saved in 'trimming/$asm.trimmedReads.fasta.gz'\n";
-
     stopAfter("readTrimming");
 }
