@@ -35,13 +35,17 @@ require Exporter;
 @EXPORT = qw(mmapConfigure mmapPrecomputeCheck mmapCheck);
 
 use strict;
+use warnings "all";
+no  warnings "uninitialized";
 
 use File::Path 2.08 qw(make_path remove_tree);
 
 use canu::Defaults;
 use canu::Execution;
-use canu::Gatekeeper;
-use canu::HTML;
+
+use canu::SequenceStore;
+use canu::Report;
+
 use canu::Grid_Cloud;
 
 #  Map long reads to long reads with minimap.
@@ -63,25 +67,48 @@ sub mmapConfigure ($$$) {
 
     caFailure("invalid type '$typ'", undef)  if (($typ ne "partial") && ($typ ne "normal"));
 
-    goto allDone   if (skipStage($asm, "$tag-mmapConfigure") == 1);
     goto allDone   if (fileExists("$path/precompute.sh")) && (fileExists("$path/mmap.sh"));
     goto allDone   if (fileExists("$path/ovljob.files"));
     goto allDone   if (-e "$base/$asm.ovlStore");
     goto allDone   if (fileExists("$base/$asm.ovlStore.tar"));
 
+    my $numPacBioRaw         = 0;
+    my $numPacBioCorrected   = 0;
+    my $numNanoporeRaw       = 0;
+    my $numNanoporeCorrected = 0;
+
+    open(L, "< ./$asm.seqStore/libraries.txt") or caExit("can't open './$asm.seqStore/libraries.txt' for reading: $!", undef);
+    while (<L>) {
+        $numPacBioRaw++           if (m/pacbio-raw/);
+        $numPacBioCorrected++     if (m/pacbio-corrected/);
+        $numNanoporeRaw++         if (m/nanopore-raw/);
+        $numNanoporeCorrected++   if (m/nanopore-corrected/);
+    }
+    close(L);
+    my $parameters = "";
+    if ($numPacBioRaw > 0) {
+       $parameters = "-x ava-pb";
+    } elsif ($numNanoporeRaw > 0) {
+       $parameters = "-x ava-ont";
+    } elsif ($numPacBioCorrected > 0) {
+       $parameters = "-x ava-pb"; # -Hk21 -w14"; #tuned to find 1000bp 5% error
+    } elsif ($numNanoporeCorrected > 0) {
+       $parameters = "-x ava-ont"; # -k17 -w11"; #tuned to find 1000bp 15% error
+    } else {
+       caFailiure("--ERROR: no know read types found in $base/$asm.seqStore/libraries.txt")
+    }
+
     print STDERR "--\n";
-    print STDERR "-- OVERLAPPER (mmap) (correction)\n"  if ($tag eq "cor");
-    print STDERR "-- OVERLAPPER (mmap) (trimming)\n"    if ($tag eq "obt");
-    print STDERR "-- OVERLAPPER (mmap) (assembly)\n"    if ($tag eq "utg");
+    print STDERR "-- OVERLAPPER (mmap) (correction) with $parameters\n"  if ($tag eq "cor");
+    print STDERR "-- OVERLAPPER (mmap) (trimming) with $parameters\n"    if ($tag eq "obt");
+    print STDERR "-- OVERLAPPER (mmap) (assembly) with $parameters\n"    if ($tag eq "utg");
     print STDERR "--\n";
 
     make_path($path) if (! -d $path);
 
     #  Constants.
 
-    my $merSize       = getGlobal("${tag}MMapMerSize");
-
-    my $numReads      = getNumberOfReadsInStore($base, $asm);
+    my $numReads      = getNumberOfReadsInStore($asm, "all");   #  Need to iterate over all read IDs!
     my $memorySize    = getGlobal("${tag}mmapMemory");
     my $blockPerGb    = getGlobal("${tag}MMapBlockSize");
     my $blockSize = int($blockPerGb * $memorySize);
@@ -198,7 +225,7 @@ sub mmapConfigure ($$$) {
     runCommandSilently($path, "tar -cf queries.tar queries", 1);
     stashFile("$path/queries.tar");
 
-    #  Create a script to generate precomputed blocks, including extracting the reads from gkpStore.
+    #  Create a script to generate precomputed blocks, including extracting the reads from seqStore.
 
     #OPTIMIZE
     #OPTIMIZE  Probably a big optimization for cloud assemblies, the block fasta inputs can be
@@ -212,7 +239,7 @@ sub mmapConfigure ($$$) {
     print F getBinDirectoryShellCode();
     print F "\n";
     print F setWorkDirectoryShellCode($path);
-    print F fetchStoreShellCode("$base/$asm.gkpStore", "$base/1-overlapper", "");
+    print F fetchSeqStoreShellCode($asm, $path, "");
     print F "\n";
     print F getJobIDShellCode();
     print F "\n";
@@ -233,13 +260,14 @@ sub mmapConfigure ($$$) {
     print F "  mkdir -p ./blocks\n";
     print F "fi\n";
     print F "\n";
-    print F fileExistsShellCode("./blocks/\$job.fasta");
+    print F fileExistsShellCode("exists", $path, "blocks/\$job.mmi");
+    print F "if [ \$exists = true ] ; then\n";
     print F "  echo Job previously completed successfully.\n";
     print F "  exit\n";
     print F "fi\n";
     print F "\n";
-    print F "\$bin/gatekeeperDumpFASTQ \\\n";
-    print F "  -G ../$asm.gkpStore \\\n";
+    print F "\$bin/sqStoreDumpFASTQ \\\n";
+    print F "  -S ../../$asm.seqStore \\\n";
     print F "  \$rge \\\n";
     print F "  -nolibname \\\n";
     print F "  -noreadname \\\n";
@@ -247,8 +275,29 @@ sub mmapConfigure ($$$) {
     print F "  -o ./blocks/\$job.input \\\n";
     print F "&& \\\n";
     print F "mv -f ./blocks/\$job.input.fasta ./blocks/\$job.fasta\n";
+    print F "if [ ! -e ./blocks/\$job.fasta ] ; then\n";
+    print F "  echo Failed to extract fasta.\n";
+    print F "  exit 1\n";
+    print F "fi\n";
     print F "\n";
-    print F stashFileShellCode("$base/1-overlapper/blocks", "\$job.fasta", "");
+    print F "\n";
+    print F "echo \"\"\n";
+    print F "echo Starting minimap precompute.\n";
+    print F "echo \"\"\n";
+    print F "\n";
+    print F "\$bin/minimap2 \\\n";
+    print F "  $parameters -t ", getGlobal("${tag}mmapThreads"), " \\\n";
+    print F "  -d ./blocks/\$job.input.mmi\\\n";
+    print F "  ./blocks/\$job.fasta \\\n";
+    print F "&& \\\n";
+    print F "mv -f ./blocks/\$job.input.mmi ./blocks/\$job.mmi\n";
+    print F "\n";
+    print F "if [ ! -e ./blocks/\$job.mmi ] ; then\n";
+    print F "  echo minimap failed.\n";
+    print F "  exit 1\n";
+    print F "fi\n";
+    print F "\n";
+    print F stashFileShellCode("$base/1-overlapper/blocks", "\$job.mmi", "");
     print F "\n";
     print F "exit 0\n";
 
@@ -263,7 +312,7 @@ sub mmapConfigure ($$$) {
     print F getBinDirectoryShellCode();
     print F "\n";
     print F setWorkDirectoryShellCode($path);
-    print F fetchStoreShellCode("$base/$asm.gkpStore", "$base/1-overlapper", "");
+    print F fetchSeqStoreShellCode($asm, $path, "");
     print F "\n";
     print F getJobIDShellCode();
     print F "\n";
@@ -300,7 +349,7 @@ sub mmapConfigure ($$$) {
     print F "fi\n";
     print F "\n";
 
-    print F fetchFileShellCode("$path", "blocks/\$blk.fasta", "");
+    print F fetchFileShellCode("$path", "blocks/\$blk.mmi", "");
 
     print F "for ii in `ls ./queries/\$qry` ; do\n";
     print F "  echo Fetch blocks/\$ii\n";
@@ -314,13 +363,9 @@ sub mmapConfigure ($$$) {
     print F "if [ x\$slf = x ]; then\n";
     print F "   >  ./results/\$qry.mmap.WORKING\n";
     print F "else\n";
-    print F "  \$bin/minimap \\\n";
-    print F "    -k $merSize \\\n";
-    print F "    -Sw5 \\\n";
-    print F "     -L100 \\\n";
-    print F "     -m0  \\\n";
-    print F "    -t ", getGlobal("${tag}mmapThreads"), " \\\n";
-    print F "    ./blocks/\$blk.fasta \\\n";
+    print F "  \$bin/minimap2 \\\n";
+    print F "    $parameters -t ", getGlobal("${tag}mmapThreads"), " \\\n";
+    print F "    ./blocks/\$blk.mmi \\\n";
     print F "    ./blocks/\$blk.fasta \\\n";
     print F "  > ./results/\$qry.mmap.WORKING \n";
     print F " \n";
@@ -328,13 +373,9 @@ sub mmapConfigure ($$$) {
     print F "\n";
 
     print F "for file in `ls queries/\$qry/*.fasta`; do\n";
-    print F "  \$bin/minimap \\\n";
-    print F "    -k $merSize \\\n";
-    print F "    -Sw5 \\\n";
-    print F "     -L100 \\\n";
-    print F "     -m0  \\\n";
-    print F "    -t ", getGlobal("${tag}mmapThreads"), " \\\n";
-    print F "    ./blocks/\$blk.fasta \\\n";
+    print F "  \$bin/minimap2 \\\n";
+    print F "    $parameters -t ", getGlobal("${tag}mmapThreads"), " \\\n";
+    print F "    ./blocks/\$blk.mmi \\\n";
     print F "    \$file \\\n";
     print F "  >> ./results/\$qry.mmap.WORKING \n";
     print F "done\n";
@@ -345,8 +386,11 @@ sub mmapConfigure ($$$) {
     print F "if [   -e ./results/\$qry.mmap -a \\\n";
     print F "     ! -e ./results/\$qry.ovb ] ; then\n";
     print F "  \$bin/mmapConvert \\\n";
-    print F "    -G ../$asm.gkpStore \\\n";
+    print F "    -S ../../$asm.seqStore \\\n";
     print F "    -o ./results/\$qry.mmap.ovb.WORKING \\\n";
+    print F "    -e " . getGlobal("${tag}OvlErrorRate");
+    print F "    -partial \\\n"  if ($typ eq "partial");
+    print F "    -len "  , getGlobal("minOverlapLength"),  " \\\n";
     print F "    ./results/\$qry.mmap \\\n";
     print F "  && \\\n";
     print F "  mv ./results/\$qry.mmap.ovb.WORKING ./results/\$qry.mmap.ovb\n";
@@ -364,7 +408,7 @@ sub mmapConfigure ($$$) {
     print F "if [ -e ./results/\$qry.mmap.ovb ] ; then\n";
     if (getGlobal("${tag}ReAlign") eq "1") {
         print F "  \$bin/overlapPair \\\n";
-        print F "    -G ../$asm.gkpStore \\\n";
+        print F "    -S ../../$asm.seqStore \\\n";
         print F "    -O ./results/\$qry.mmap.ovb \\\n";
         print F "    -o ./results/\$qry.ovb \\\n";
         print F "    -partial \\\n"  if ($typ eq "partial");
@@ -374,13 +418,13 @@ sub mmapConfigure ($$$) {
         print F "    -memory " . getGlobal("${tag}mmapMemory") . " \\\n";
         print F "    -t " . getGlobal("${tag}mmapThreads") . " \n";
     } else {
-        print F "  mv -f ./results/\$qry.mmap.ovb    ./results/\$qry.ovb\n";
-        print F "  mv -f ./results/\$qry.mmap.counts ./results/\$qry.counts\n";
+        print F "  mv -f ./results/\$qry.mmap.ovb ./results/\$qry.ovb\n";
+        print F "  mv -f ./results/\$qry.mmap.oc  ./results/\$qry.oc\n";
     }
     print F "fi\n";
 
-    print F stashFileShellCode("$path", "results/\$qry.ovb",    "");
-    print F stashFileShellCode("$path", "results/\$qry.counts", "");
+    print F stashFileShellCode("$path", "results/\$qry.ovb", "");
+    print F stashFileShellCode("$path", "results/\$qry.oc",  "");
     print F "\n";
 
     print F "\n";
@@ -410,12 +454,15 @@ sub mmapConfigure ($$$) {
         print STDERR "-- Configured $numJobs mmap overlap jobs.\n";
     }
 
+    makeExecutable("$path/precompute.sh");
+    makeExecutable("$path/mhap.sh");
+
     stashFile("$path/precompute.sh");
     stashFile("$path/mhap.sh");
 
   finishStage:
-    emitStage($asm, "$tag-mmapConfigure");
-    buildHTML($asm, $tag);
+    generateReport($asm);
+    resetIteration("$tag-mmapConfigure");
 
   allDone:
     stopAfter("overlapConfigure");
@@ -437,7 +484,6 @@ sub mmapPrecomputeCheck ($$$) {
 
     $path = "$base/1-overlapper";
 
-    goto allDone   if (skipStage($asm, "$tag-mmapPrecomputeCheck", $attempt) == 1);
     goto allDone   if (fileExists("$path/precompute.files"));
     goto allDone   if (-e "$base/$asm.ovlStore");
     goto allDone   if (fileExists("$base/$asm.ovlStore.tar"));
@@ -454,10 +500,10 @@ sub mmapPrecomputeCheck ($$$) {
     open(F, "< $path/precompute.sh") or caFailure("can't open '$path/precompute.sh' for reading: $!", undef);
     while (<F>) {
         if (m/^\s+job=\"(\d+)\"$/) {
-            if (fileExists("$path/blocks/$1.fasta")) {
-                push @successJobs, "1-overlapper/blocks/$1.fasta\n";
+            if (fileExists("$path/blocks/$1.mmi")) {
+                push @successJobs, "1-overlapper/blocks/$1.mmi\\n";
             } else {
-                $failureMessage .= "--   job 1-overlapper/blocks/$1.fasta FAILED.\n";
+                $failureMessage .= "--   job $path/blocks/$1.fasta FAILED.\n";
                 push @failedJobs, $currentJobID;
             }
 
@@ -470,25 +516,26 @@ sub mmapPrecomputeCheck ($$$) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
+        #  If too many attempts, give up.
 
-        if ($attempt > 1) {
+        if ($attempt >= getGlobal("canuIterationMax")) {
             print STDERR "--\n";
-            print STDERR "-- ", scalar(@failedJobs), " mmap precompute jobs failed:\n";
+            print STDERR "-- MiniMap precompute jobs failed, tried $attempt times, giving up.\n";
+            print STDERR $failureMessage;
+            print STDERR "--\n";
+            caExit(undef, undef);
+        }
+
+        if ($attempt > 0) {
+            print STDERR "--\n";
+            print STDERR "-- MiniMap precompute jobs failed, retry.\n";
             print STDERR $failureMessage;
             print STDERR "--\n";
         }
 
-        #  If too many attempts, give up.
-
-        if ($attempt > getGlobal("canuIterationMax")) {
-            caExit("failed to precompute mmap indices.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
-        }
-
         #  Otherwise, run some jobs.
 
-        emitStage($asm, "$tag-mmapPrecomputeCheck", $attempt);
-        buildHTML($asm, $tag);
+        generateReport($asm);
 
         submitOrRunParallelJob($asm, "${tag}mmap", $path, "precompute", @failedJobs);
         return;
@@ -503,8 +550,8 @@ sub mmapPrecomputeCheck ($$$) {
 
     stashFile("$path/precompute.files");
 
-    emitStage($asm, "$tag-mmapPrecomputeCheck");
-    buildHTML($asm, $tag);
+    generateReport($asm);
+    resetIteration("$tag-mmapPrecomputeCheck");
 
   allDone:
 }
@@ -526,7 +573,6 @@ sub mmapCheck ($$$) {
 
     $path = "$base/1-overlapper";
 
-    goto allDone   if (skipStage($asm, "$tag-mmapCheck", $attempt) == 1);
     goto allDone   if (fileExists("$path/mmap.files"));
     goto allDone   if (-e "$base/$asm.ovlStore");
     goto allDone   if (fileExists("$base/$asm.ovlStore.tar"));
@@ -549,28 +595,28 @@ sub mmapCheck ($$$) {
                 push @mmapJobs,    "1-overlapper/results/$1.mmap\n";
                 push @successJobs, "1-overlapper/results/$1.ovb.gz\n";
                 push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+                push @miscJobs,    "1-overlapper/results/$1.oc\n";
 
             } elsif (fileExists("$path/results/$1.ovb")) {
                 push @mmapJobs,    "1-overlapper/results/$1.mmap\n";
                 push @successJobs, "1-overlapper/results/$1.ovb\n";
                 push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+                push @miscJobs,    "1-overlapper/results/$1.oc\n";
 
             } elsif (fileExists("$path/results/$1.ovb.bz2")) {
                 push @mmapJobs,    "1-overlapper/results/$1.mmap\n";
                 push @successJobs, "1-overlapper/results/$1.ovb.bz2\n";
                 push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+                push @miscJobs,    "1-overlapper/results/$1.oc\n";
 
             } elsif (fileExists("$path/results/$1.ovb.xz")) {
                 push @mmapJobs,    "1-overlapper/results/$1.mmap\n";
                 push @successJobs, "1-overlapper/results/$1.ovb.xz\n";
                 push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+                push @miscJobs,    "1-overlapper/results/$1.oc\n";
 
             } else {
-                $failureMessage .= "--   job 1-overlapper/results/$1.ovb FAILED.\n";
+                $failureMessage .= "--   job $path/results/$1.ovb FAILED.\n";
                 push @failedJobs, $currentJobID;
             }
 
@@ -582,7 +628,7 @@ sub mmapCheck ($$$) {
     #  Also find the queries symlinks so we can remove those.  And the query directories, because
     #  the last directory can be empty, and so we'd never see it at all if only finding files.
 
-    open(F, "find $path/queries -print |");
+    open(F, "cd $base && find 1-overlapper/queries -print |");
     while (<F>) {
         push @mmapJobs, $_;
     }
@@ -592,25 +638,27 @@ sub mmapCheck ($$$) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
+        #  If too many attempts, give up.
 
-        if ($attempt > 1) {
+        if ($attempt >= getGlobal("canuIterationMax")) {
             print STDERR "--\n";
-            print STDERR "-- ", scalar(@failedJobs), " mmap jobs failed:\n";
+            print STDERR "-- MiniMap overlap jobs failed, tried $attempt times, giving up.\n";
+            print STDERR $failureMessage;
+            print STDERR "--\n";
+            caExit(undef, undef);
+        }
+
+        if ($attempt > 0) {
+            print STDERR "--\n";
+            print STDERR "-- MiniMap overlap jobs failed, retry.\n";
             print STDERR $failureMessage;
             print STDERR "--\n";
         }
 
-        #  If too many attempts, give up.
-
-        if ($attempt > getGlobal("canuIterationMax")) {
-            caExit("failed to compute mmap overlaps.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
-        }
-
         #  Otherwise, run some jobs.
 
-        emitStage($asm, "$tag-mmapCheck", $attempt);
-        buildHTML($asm, $tag);
+        generateReport($asm);
+
         submitOrRunParallelJob($asm, "${tag}mmap", $path, "mmap", @failedJobs);
         return;
     }
@@ -634,8 +682,8 @@ sub mmapCheck ($$$) {
     stashFile("$path/ovljob.files");
     stashFile("$path/ovljob.more.files");
 
-    emitStage($asm, "$tag-mmapCheck");
-    buildHTML($asm, $tag);
+    generateReport($asm);
+    resetIteration("$tag-mmapCheck");
 
   allDone:
     stopAfter("overlap");
