@@ -43,14 +43,18 @@ require Exporter;
 @EXPORT = qw(mhapConfigure mhapPrecomputeCheck mhapCheck);
 
 use strict;
+use warnings "all";
+no  warnings "uninitialized";
 use POSIX qw(floor);
 
 use File::Path 2.08 qw(make_path remove_tree);
 
 use canu::Defaults;
 use canu::Execution;
-use canu::Gatekeeper;
-use canu::HTML;
+
+use canu::SequenceStore;
+use canu::Report;
+
 use canu::Grid_Cloud;
 
 #  Map long reads to long reads with mhap.
@@ -78,7 +82,6 @@ sub mhapConfigure ($$$) {
 
     caFailure("invalid type '$typ'", undef)  if (($typ ne "partial") && ($typ ne "normal"));
 
-    goto allDone   if (skipStage($asm, "$tag-mhapConfigure") == 1);
     goto allDone   if (fileExists("$path/precompute.sh")) && (fileExists("$path/mhap.sh"));
     goto allDone   if (fileExists("$path/ovljob.files"));
     goto allDone   if (-e "$base/$asm.ovlStore");
@@ -97,7 +100,7 @@ sub mhapConfigure ($$$) {
     my ($numHashes, $minNumMatches, $threshold, $ordSketch, $ordSketchMer);
 
     if (!defined(getGlobal("${tag}MhapSensitivity"))) {
-        my $cov = getExpectedCoverage($base, $asm);
+        my $cov = getExpectedCoverage($tag, $asm);
 
         setGlobal("${tag}MhapSensitivity", "low");                          #  Yup, super inefficient.  The code is
         setGlobal("${tag}MhapSensitivity", "normal")   if ($cov <  60);     #  compact and clear and runs once.
@@ -131,9 +134,17 @@ sub mhapConfigure ($$$) {
         caFailure("invalid ${tag}MhapSensitivity=" . getGlobal("${tag}MhapSensitivity"), undef);
     }
 
+    # quick guess parameter adjustment for corrected reads, hack for now and should better take error rate into account
+    if (($tag eq "obt") || ($tag eq "utg")) {
+       $numHashes      = "128";
+       $minNumMatches  = 5;
+       $ordSketch      = 1000;
+       $threshold      = 1-getGlobal("${tag}OvlErrorRate");
+    }
+
     # due to systematic bias in nanopore data, adjust threshold up by 5%
     my $numNanoporeRaw = 0;
-    open(L, "< $base/$asm.gkpStore/libraries.txt") or caExit("can't open '$base/$asm.gkpStore/libraries.txt' for reading: $!", undef);
+    open(L, "< ./$asm.seqStore/libraries.txt") or caExit("can't open './$asm.seqStore/libraries.txt' for reading: $!", undef);
     while (<L>) {
         $numNanoporeRaw++         if (m/nanopore-raw/);
     }
@@ -147,20 +158,13 @@ sub mhapConfigure ($$$) {
 
     my $merSize       = getGlobal("${tag}MhapMerSize");
 
-    my $numReads      = getNumberOfReadsInStore($base, $asm);
-    my $memorySize    = getGlobal("${tag}mhapMemory");
-    my $blockPerGb    = getGlobal("${tag}MhapBlockSize");
-    if ($numHashes >= 768) {
-       $blockPerGb = int($blockPerGb / 2);
-    }
+    my $numReads      = getNumberOfReadsInStore($asm, "all");   #  Need to iterate over all read IDs!
+    my $memorySize    = getGlobal("${tag}mhapMemory") * 0.9;
+    my $blockPerGb    = getGlobal("${tag}MhapBlockSize") / (($numHashes < 768) ? 1 : 2);
 
-    # quick guess parameter adjustment for corrected reads, hack for now and should better take error rate into account
-    if (($tag eq "obt") || ($tag eq "utg")) {
-       $numHashes      = "128";
-       $minNumMatches  = 5;
-       $ordSketch      = 1000;
-       $threshold      = 1-getGlobal("${tag}OvlErrorRate");
-    }
+    my $javaPath      = getGlobal("java");
+    my $javaOpt       = "-d64" if (defined(getGlobal("javaUse64Bit")) && getGlobal("javaUse64Bit") == 1);
+    my $javaMemory    = int(getGlobal("${tag}mhapMemory") * 1024 * 0.9 + 0.5);
 
     print STDERR "--\n";
     print STDERR "-- PARAMETERS: hashes=$numHashes, minMatches=$minNumMatches, threshold=$threshold\n";
@@ -179,14 +183,12 @@ sub mhapConfigure ($$$) {
 
     my @hashes;    #  One for each job, the block that is the hash table
     my @skipSelf;  #  One for each job, jobs that would search block N against hash block N need to be handled special
-    my @convert;   #  One for each job, flags to the mhap-ovl conversion program
 
     push @blocks,   "no zeroth block, makes the loop where this is used easier";
     push @blockBgn, "no zeroth block";
     push @blockLen, "no zeroth block";
     push @hashes,   "no zeroth job";
     push @skipSelf, "no zeroth job";
-    push @convert,  "no zeroth job";
 
     for (my $bgn=1; $bgn < $numReads; $bgn += $blockSize) {
         my $end = $bgn + $blockSize - 1;
@@ -283,15 +285,6 @@ sub mhapConfigure ($$$) {
             #  This is easy, the ID of the hash.
 
             push @hashes, substr("000000" . $bid, -6);  #  One new job for block bid with qend-qbgn query files in it
-
-            #  Annoyingly, if we're against 'self', then the conversion needs to know that the query IDs
-            #  aren't offset by the number of hash reads.
-
-            if ($andSelf eq "") {
-                push @convert, "-h $blockBgn[$bid] $blockLen[$bid] -q $blockBgn[$qbgn]";
-            } else {
-                push @convert, "-h $blockBgn[$bid] 0 -q $blockBgn[$bid]";
-            }
         }
     }
 
@@ -302,16 +295,13 @@ sub mhapConfigure ($$$) {
     runCommandSilently($path, "tar -cf queries.tar queries", 1);
     stashFile("$path/queries.tar");
 
-    #  Create a script to generate precomputed blocks, including extracting the reads from gkpStore.
+    #  Create a script to generate precomputed blocks, including extracting the reads from seqStore.
 
     #OPTIMIZE
     #OPTIMIZE  Probably a big optimization for cloud assemblies, the block fasta inputs can be
     #OPTIMIZE  computed ahead of time, stashed, and then fetched to do the actual precompute.
     #OPTIMIZE
 
-    my $javaPath   = getGlobal("java");
-    my $javaMemory = int(getGlobal("${tag}mhapMemory") * 1024 + 0.5);
-
     my $cygA       = ($^O ne "cygwin") ? "" : "\$(cygpath -w ";   #  Some baloney to convert nice POSIX paths into
     my $cygB       = ($^O ne "cygwin") ? "" : ")";                #  whatever abomination Windows expects.
 
@@ -322,7 +312,7 @@ sub mhapConfigure ($$$) {
     print F getBinDirectoryShellCode();
     print F "\n";
     print F setWorkDirectoryShellCode($path);
-    print F fetchStoreShellCode("$base/$asm.gkpStore", "$base/1-overlapper", "");
+    print F fetchSeqStoreShellCode($asm, $path, "");
     print F "\n";
     print F getJobIDShellCode();
     print F "\n";
@@ -343,7 +333,8 @@ sub mhapConfigure ($$$) {
     print F "  mkdir -p ./blocks\n";
     print F "fi\n";
     print F "\n";
-    print F fileExistsShellCode("./blocks/\$job.dat");
+    print F fileExistsShellCode("exists", "$path", "blocks/\$job.dat");
+    print F "if [ \$exists = true ] ; then\n";
     print F "  echo Job previously completed successfully.\n";
     print F "  exit\n";
     print F "fi\n";
@@ -351,12 +342,11 @@ sub mhapConfigure ($$$) {
     print F "#  Remove any previous result.\n";
     print F "rm -f ./blocks/\$job.input.dat\n";
     print F "\n";
-    #print F fetchFileShellCode("./blocks/\$job.input.fasta");
-    print F "\n";
-    print F "\$bin/gatekeeperDumpFASTQ \\\n";
-    print F "  -G ../$asm.gkpStore \\\n";
+    print F "\$bin/sqStoreDumpFASTQ \\\n";
+    print F "  -S ../../$asm.seqStore \\\n";
     print F "  \$rge \\\n";
     print F "  -nolibname \\\n";
+    print F "  -noreadname \\\n";
     print F "  -fasta \\\n";
     print F "  -o ./blocks/\$job.input \\\n";
     print F "|| \\\n";
@@ -368,7 +358,7 @@ sub mhapConfigure ($$$) {
     print F "  exit 1\n";
     print F "fi\n";
     print F "\n";
-    print F fetchFileShellCode("$base/0-mercounts", "$asm.ms$merSize.frequentMers.ignore.gz", "");
+    print F fetchFileShellCode("$base/0-mercounts", "$asm.ms$merSize.ignore.gz", "");
     print F "\n";
     print F "echo \"\"\n";
     print F "echo Starting mhap precompute.\n";
@@ -377,11 +367,12 @@ sub mhapConfigure ($$$) {
     print F "#  So mhap writes its output in the correct spot.\n";
     print F "cd ./blocks\n";
     print F "\n";
-    print F "$javaPath -d64 -server -Xmx", $javaMemory, "m \\\n";
-    print F "  -jar $cygA \$bin/mhap-" . getGlobal("${tag}MhapVersion") . ".jar $cygB \\\n";
+    print F "$javaPath $javaOpt -XX:ParallelGCThreads=",  getGlobal("${tag}mhapThreads"), " -server -Xms", $javaMemory, "m -Xmx", $javaMemory, "m \\\n";
+    print F "  -jar $cygA \$bin/../share/java/classes/mhap-" . getGlobal("${tag}MhapVersion") . ".jar $cygB \\\n";
     print F "  --repeat-weight 0.9 --repeat-idf-scale 10 -k $merSize \\\n";
     print F "  --supress-noise 2 \\\n"  if (defined(getGlobal("${tag}MhapFilterUnique")) && getGlobal("${tag}MhapFilterUnique") == 1);
     print F "  --no-tf \\\n"            if (defined(getGlobal("${tag}MhapNoTf")) && getGlobal("${tag}MhapNoTf") == 1);
+    print F "  --store-full-id \\\n";
     print F "  --num-hashes $numHashes \\\n";
     print F "  --num-min-matches $minNumMatches \\\n";
     print F "  --ordered-sketch-size $ordSketch \\\n";
@@ -391,7 +382,7 @@ sub mhapConfigure ($$$) {
     print F "  --min-olap-length ", getGlobal("minOverlapLength"), " \\\n";
     print F "  --num-threads ", getGlobal("${tag}mhapThreads"), " \\\n";
     print F " " . getGlobal("${tag}MhapOptions")         . " \\\n"   if (defined(getGlobal("${tag}MhapOptions")));
-    print F "  -f $cygA ../../0-mercounts/$asm.ms$merSize.frequentMers.ignore.gz $cygB \\\n"   if (-e "$base/0-mercounts/$asm.ms$merSize.frequentMers.ignore.gz");
+    print F "  -f $cygA ../../0-mercounts/$asm.ms$merSize.ignore.gz $cygB \\\n"   if (-e "$base/0-mercounts/$asm.ms$merSize.ignore.gz");
     print F "  -p $cygA ./\$job.input.fasta $cygB \\\n";
     print F "  -q $cygA . $cygB \\\n";
     print F "&& \\\n";
@@ -420,7 +411,7 @@ sub mhapConfigure ($$$) {
     print F getBinDirectoryShellCode();
     print F "\n";
     print F setWorkDirectoryShellCode($path);
-    print F fetchStoreShellCode("$base/$asm.gkpStore", "$base/1-overlapper", "");
+    print F fetchSeqStoreShellCode($asm, $path, "");
     print F "\n";
     print F getJobIDShellCode();
     print F "\n";
@@ -428,7 +419,6 @@ sub mhapConfigure ($$$) {
         print F "if [ \$jobid -eq $ii ] ; then\n";
         print F "  blk=\"$hashes[$ii]\"\n";
         print F "  slf=\"$skipSelf[$ii]\"\n";
-        print F "  cvt=\"$convert[$ii]\"\n";
         print F "  qry=\"", substr("000000" . $ii, -6), "\"\n";
         print F "fi\n";
         print F "\n";
@@ -477,11 +467,12 @@ sub mhapConfigure ($$$) {
     print F "echo \"\"\n";
     print F "\n";
     print F "if [ ! -e ./results/\$qry.mhap ] ; then\n";
-    print F "  $javaPath -d64 -server -Xmx", $javaMemory, "m \\\n";
-    print F "    -jar $cygA \$bin/mhap-" . getGlobal("${tag}MhapVersion") . ".jar $cygB \\\n";
+    print F "  $javaPath $javaOpt -XX:ParallelGCThreads=",  getGlobal("${tag}mhapThreads"), " -server -Xms", $javaMemory, "m -Xmx", $javaMemory, "m \\\n";
+    print F "    -jar $cygA \$bin/../share/java/classes/mhap-" . getGlobal("${tag}MhapVersion") . ".jar $cygB \\\n";
     print F "    --repeat-weight 0.9 --repeat-idf-scale 10 -k $merSize \\\n";
     print F "    --supress-noise 2 \\\n"  if (defined(getGlobal("${tag}MhapFilterUnique")) && getGlobal("${tag}MhapFilterUnique") == 1);
     print F "    --no-tf \\\n"            if (defined(getGlobal("${tag}MhapNoTf")) && getGlobal("${tag}MhapNoTf") == 1);
+    print F "    --store-full-id \\\n";
     print F "    --num-hashes $numHashes \\\n";
     print F "    --num-min-matches $minNumMatches \\\n";
     print F "    --threshold $threshold \\\n";
@@ -502,8 +493,7 @@ sub mhapConfigure ($$$) {
     print F "if [   -e ./results/\$qry.mhap -a \\\n";
     print F "     ! -e ./results/\$qry.ovb ] ; then\n";
     print F "  \$bin/mhapConvert \\\n";
-    print F "    -G ../$asm.gkpStore \\\n";
-    print F "    \$cvt \\\n";
+    print F "    -S ../../$asm.seqStore \\\n";
     print F "    -o ./results/\$qry.mhap.ovb.WORKING \\\n";
     print F "    ./results/\$qry.mhap \\\n";
     print F "  && \\\n";
@@ -522,7 +512,7 @@ sub mhapConfigure ($$$) {
     if (getGlobal("${tag}ReAlign") eq "1") {
         print F "if [ -e ./results/\$qry.mhap.ovb ] ; then\n";
         print F "  \$bin/overlapPair \\\n";
-        print F "    -G ../$asm.gkpStore \\\n";
+        print F "    -S ../../$asm.seqStore \\\n";
         print F "    -O ./results/\$qry.mhap.ovb \\\n";
         print F "    -o ./results/\$qry.WORKING.ovb \\\n";
         print F "    -partial \\\n"  if ($typ eq "partial");
@@ -542,8 +532,8 @@ sub mhapConfigure ($$$) {
     }
 
     print F "\n";
-    print F stashFileShellCode("$path", "results/\$qry.ovb",    "");
-    print F stashFileShellCode("$path", "results/\$qry.counts", "");
+    print F stashFileShellCode("$path", "results/\$qry.ovb", "");
+    print F stashFileShellCode("$path", "results/\$qry.oc",  "");
     print F "\n";
     print F "exit 0\n";
 
@@ -578,8 +568,8 @@ sub mhapConfigure ($$$) {
     stashFile("$path/mhap.sh");
 
   finishStage:
-    emitStage($asm, "$tag-mhapConfigure");
-    buildHTML($asm, $tag);
+    generateReport($asm);
+    resetIteration("$tag-mhapConfigure");
 
   allDone:
     stopAfter("overlapConfigure");
@@ -603,7 +593,6 @@ sub mhapPrecomputeCheck ($$$) {
 
     $path = "$base/1-overlapper";
 
-    goto allDone   if (skipStage($asm, "$tag-mhapPrecomputeCheck", $attempt) == 1);
     goto allDone   if (fileExists("$path/precompute.files"));
     goto allDone   if (-e "$base/$asm.ovlStore");
     goto allDone   if (fileExists("$base/$asm.ovlStore.tar"));
@@ -655,8 +644,7 @@ sub mhapPrecomputeCheck ($$$) {
 
         #  Otherwise, run some jobs.
 
-        emitStage($asm, "$tag-mhapPrecomputeCheck", $attempt);
-        buildHTML($asm, $tag);
+        generateReport($asm);
 
         submitOrRunParallelJob($asm, "${tag}mhap", $path, "precompute", @failedJobs);
         return;
@@ -671,8 +659,8 @@ sub mhapPrecomputeCheck ($$$) {
 
     stashFile("$path/precompute.files");
 
-    emitStage($asm, "$tag-mhapPrecomputeCheck");
-    buildHTML($asm, $tag);
+    generateReport($asm);
+    resetIteration("$tag-mhapPrecomputeCheck");
 
   allDone:
 }
@@ -697,7 +685,6 @@ sub mhapCheck ($$$) {
 
     $path = "$base/1-overlapper";
 
-    goto allDone   if (skipStage($asm, "$tag-mhapCheck", $attempt) == 1);
     goto allDone   if (fileExists("$path/mhap.files"));
     goto allDone   if (-e "$base/$asm.ovlStore");
     goto allDone   if (fileExists("$base/$asm.ovlStore.tar"));
@@ -720,25 +707,25 @@ sub mhapCheck ($$$) {
                 push @mhapJobs,    "1-overlapper/results/$1.mhap\n";
                 push @successJobs, "1-overlapper/results/$1.ovb.gz\n";
                 push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+                push @miscJobs,    "1-overlapper/results/$1.oc\n";
 
             } elsif (fileExists("$path/results/$1.ovb")) {
                 push @mhapJobs,    "1-overlapper/results/$1.mhap\n";
                 push @successJobs, "1-overlapper/results/$1.ovb\n";
                 push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+                push @miscJobs,    "1-overlapper/results/$1.oc\n";
 
             } elsif (fileExists("$path/results/$1.ovb.bz2")) {
                 push @mhapJobs,    "1-overlapper/results/$1.mhap\n";
                 push @successJobs, "1-overlapper/results/$1.ovb.bz2\n";
                 push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+                push @miscJobs,    "1-overlapper/results/$1.oc\n";
 
             } elsif (fileExists("$path/results/$1.ovb.xz")) {
                 push @mhapJobs,    "1-overlapper/results/$1.mhap\n";
                 push @successJobs, "1-overlapper/results/$1.ovb.xz\n";
                 push @miscJobs,    "1-overlapper/results/$1.stats\n";
-                push @miscJobs,    "1-overlapper/results/$1.counts\n";
+                push @miscJobs,    "1-overlapper/results/$1.oc\n";
 
             } else {
                 $failureMessage .= "--   job $path/results/$1.ovb FAILED.\n";
@@ -782,8 +769,8 @@ sub mhapCheck ($$$) {
 
         #  Otherwise, run some jobs.
 
-        emitStage($asm, "$tag-mhapCheck", $attempt);
-        buildHTML($asm, $tag);
+        generateReport($asm);
+
         submitOrRunParallelJob($asm, "${tag}mhap", $path, "mhap", @failedJobs);
         return;
     }
@@ -807,8 +794,8 @@ sub mhapCheck ($$$) {
     stashFile("$path/ovljob.files");
     stashFile("$path/ovljob.more.files");
 
-    emitStage($asm, "$tag-mhapCheck");
-    buildHTML($asm, $tag);
+    generateReport($asm);
+    resetIteration("$tag-mhapCheck");
 
   allDone:
     stopAfter("overlap");
