@@ -31,6 +31,8 @@ require Exporter;
 @EXPORT = qw(generateReport addToReport getFromReport);
 
 use strict;
+use warnings "all";
+no  warnings "uninitialized";
 
 #use File::Copy;
 #use File::Path 2.08 qw(make_path remove_tree);
@@ -72,10 +74,11 @@ sub loadReport ($) {
             $tag = "utg";
         }
 
-        if      (m/READS\]$/)        {  $rpt = "${tag}GkpStore";  $report{$rpt} = undef;
+        if      (m/READS\]$/)        {  $rpt = "${tag}SeqStore";  $report{$rpt} = undef;
         } elsif (m/MERS\]$/)         {  $rpt = "${tag}Meryl";     $report{$rpt} = undef;
 
-        } elsif (m/FILTERING\]$/)    {  $rpt = "filtering";       $report{$rpt} = undef;
+        } elsif (m/FILTERING\]$/)    {  $rpt = "corFilter";       $report{$rpt} = undef;
+        } elsif (m/LAYOUT\]$/)       {  $rpt = "corLayout";          $report{$rpt} = undef;
         } elsif (m/CORRECTIONS\]$/)  {  $rpt = "corrections";     $report{$rpt} = undef;
 
         } elsif (m/TRIMMING\]$/)     {  $rpt = "trimming";        $report{$rpt} = undef;
@@ -119,17 +122,18 @@ sub saveReport ($) {
 
     open(F, "> $asm.report") or caExit("can't open '$asm.report' for writing: $!", undef);
 
-    saveReportItem("CORRECTION/READS",       $report{"corGkpStore"});
+    saveReportItem("CORRECTION/READS",       $report{"corSeqStore"});
     saveReportItem("CORRECTION/MERS",        $report{"corMeryl"});
-    saveReportItem("CORRECTION/FILTERING",   $report{"correctionFiltering"});
+    saveReportItem("CORRECTION/FILTERING",   $report{"corFilter"});
+    saveReportItem("CORRECTION/LAYOUT",      $report{"corLayout"});
     saveReportItem("CORRECTION/CORRECTIONS", $report{"corrections"});
 
-    saveReportItem("TRIMMING/READS",         $report{"obtGkpStore"});
+    saveReportItem("TRIMMING/READS",         $report{"obtSeqStore"});
     saveReportItem("TRIMMING/MERS",          $report{"obtMeryl"});
     saveReportItem("TRIMMING/TRIMMING",      $report{"trimming"});
     saveReportItem("TRIMMING/SPLITTING",     $report{"splitting"});
 
-    saveReportItem("UNITIGGING/READS",       $report{"utgGkpStore"});
+    saveReportItem("UNITIGGING/READS",       $report{"utgSeqStore"});
     saveReportItem("UNITIGGING/MERS",        $report{"utgMeryl"});
     saveReportItem("UNITIGGING/OVERLAPS",    $report{"overlaps"});
     saveReportItem("UNITIGGING/ADJUSTMENT",  $report{"adjustments"});
